#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import onnx
from .base_operator import QuantOperatorBase
from onnxruntime.quantization.quant_utils import QuantizedValueType
from onnx import onnx_pb as onnx_proto
from lpot.adaptor.ox_utils.util import QuantizedValue


class QMaxPool(QuantOperatorBase):
    def __init__(self, onnx_quantizer, onnx_node):
        super().__init__(onnx_quantizer, onnx_node)

    def quantize(self):
        node = self.node
        assert (node.op_type == "MaxPool")

        if self.quantizer.opset_version < 12:
            super().quantize()
            return

        # When mode is QLinearOps, the output quantization params are calculated 
        # based on outputs from activation nodes, therefore these nodes can be 
        # removed from the graph if they follow a quantized op.
        # If input to this node is not quantized then keep this node
        if node.input[0] not in self.quantizer.quantized_value_map:
            self.quantizer.new_nodes += [node]
            return

        # Create an entry for output quantized value
        quantized_input_value = self.quantizer.quantized_value_map[node.input[0]]
        quantized_output_value = QuantizedValue(node.output[0], node.output[0] + "_quantized",
                                                quantized_input_value.scale_name, 
                                                quantized_input_value.zp_name,
                                                QuantizedValueType.Input)
        self.quantizer.quantized_value_map[node.output[0]] = quantized_output_value

        node.input[0] = quantized_input_value.q_name
        node.output[0] = quantized_output_value.q_name
        self.quantizer.new_nodes += [node]



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils test."""

import inspect
import os
import unittest
from unittest.mock import MagicMock, patch

from lpot.ux.utils.exceptions import AccessDeniedException, ClientErrorException, NotFoundException
from lpot.ux.utils.utils import (
    check_module,
    get_dataset_path,
    get_file_extension,
    get_framework_from_path,
    get_predefined_config_path,
    is_dataset_file,
    is_development_env,
    is_hidden,
    load_dataloader_config,
    load_help_lpot_params,
    load_model_config,
    load_transforms_config,
    verify_file_path,
)


class TestUtils(unittest.TestCase):
    """Value parser tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Parser tests constructor."""
        super().__init__(*args, **kwargs)

    def test_is_hidden(self) -> None:
        """Test if path is correctly recognized as hidden."""
        path = ".ssh"
        result = is_hidden(path)
        self.assertTrue(result)

    def test_is_not_hidden(self) -> None:
        """Test if path is correctly recognized as not hidden."""
        path = "home"
        result = is_hidden(path)
        self.assertFalse(result)

    def test_get_dataset_path(self) -> None:
        """Test getting dataset path."""
        framework = "tensorflow"
        domain = "image_recognition"
        result = get_dataset_path(framework, domain)
        self.assertEqual(result, "examples/test/dataset/imagenet")

    def test_get_dataset_path_unknown_framework(self) -> None:
        """Test getting dataset path failure."""
        framework = "unknown"
        domain = "image_recognition"
        with self.assertRaises(Exception):
            get_dataset_path(framework, domain)

    def test_get_dataset_path_unknown_domain(self) -> None:
        """Test getting dataset path failure."""
        framework = "tensorflow"
        domain = "domain"
        with self.assertRaises(Exception):
            get_dataset_path(framework, domain)

    @patch("lpot.ux.components.model.tensorflow.frozen_pb.get_model_type")
    def test_get_tensorflow_framework_from_path(self, mocked_get_model_type: MagicMock) -> None:
        """Test getting framework name from path."""
        mocked_get_model_type.return_value = "frozen_pb"
        path = "/home/user/model.pb"
        result = get_framework_from_path(path)
        self.assertEqual(result, "tensorflow")
        mocked_get_model_type.assert_called_with(path)

    def test_get_onnx_framework_from_path(self) -> None:
        """Test getting framework name from path."""
        path = "/home/user/model.onnx"
        result = get_framework_from_path(path)
        self.assertEqual(result, "onnxrt")

    def test_get_unknown_framework_from_path(self) -> None:
        """Test getting framework name from path."""
        path = "/home/user/model.some_extension"
        result = get_framework_from_path(path)
        self.assertIsNone(result)

    def test_get_file_extension(self) -> None:
        """Test getting file extension from path."""
        path = "/home/user/file.ext"
        result = get_file_extension(path)
        self.assertEqual(result, "ext")

    def test_get_file_with_dots_extension(self) -> None:
        """Test getting file extension from path."""
        path = "/home/user/file.name.ext2"
        result = get_file_extension(path)
        self.assertEqual(result, "ext2")

    def test_get_file_without_extension(self) -> None:
        """Test getting file extension from path."""
        path = "/home/user/file"
        result = get_file_extension(path)
        self.assertEqual(result, "")

    def test_record_is_dataset_file(self) -> None:
        """Test if record is a valid dataset file."""
        path = "/home/user/dataset.record"
        result = is_dataset_file(path)
        self.assertTrue(result)

    def test_pbtxt_is_dataset_file(self) -> None:
        """Test if record is a valid dataset file."""
        path = "/home/user/dataset.pbtxt"
        result = is_dataset_file(path)
        self.assertFalse(result)

    def test_get_predefined_tf_image_recognition_config_path(self) -> None:
        """Test getting predefined config path for TF image recognition models."""
        self._assert_predefined_config_path(
            framework="tensorflow",
            domain="image_recognition",
        )

    def test_get_predefined_tf_object_detection_config_path(self) -> None:
        """Test getting predefined config path for TF object detection models."""
        self._assert_predefined_config_path(
            framework="tensorflow",
            domain="object_detection",
        )

    def test_get_predefined_tf_nlp_config_path(self) -> None:
        """Test getting predefined config path for TF NLP models."""
        self._assert_predefined_config_path(
            framework="tensorflow",
            domain="nlp",
        )

    def test_get_predefined_tf_recommendation_config_path(self) -> None:
        """Test getting predefined config path for TF recommendation models."""
        self._assert_predefined_config_path(
            framework="tensorflow",
            domain="recommendation",
        )

    def test_get_predefined_onnx_image_recognition_config_path(self) -> None:
        """Test getting predefined config path for onnx image recognition models."""
        self._assert_predefined_config_path(
            framework="onnxrt",
            domain="image_recognition",
        )

    def test_get_predefined_onnx_nlp_config_path(self) -> None:
        """Test getting predefined config path for onnx NLP models."""
        self._assert_predefined_config_path(
            framework="onnxrt",
            domain="nlp",
        )

    def test_get_predefined_config_path_framework_failure(self) -> None:
        """Test getting predefined config path for onnx NLP models."""
        with self.assertRaises(Exception):
            get_predefined_config_path(
                framework="onnx",
                domain="image_recognition",
            )

    def test_get_predefined_config_path_domain_failure(self) -> None:
        """Test getting predefined config path for onnx NLP models."""
        with self.assertRaises(Exception):
            get_predefined_config_path(
                framework="onnxrt",
                domain="object_detection",
            )

    def test_check_module(self) -> None:
        """Test checking existing module."""
        check_module("os")

    def test_check_non_existing_module(self) -> None:
        """Test checking non existing module."""
        with self.assertRaises(ClientErrorException):
            check_module("non_existing_module")

    def test_load_model_config(self) -> None:
        """Test getting models config."""
        result = load_model_config()
        self.assertIs(type(result), dict)
        self.assertIsNot(result, {})

    def test_load_dataloader_config(self) -> None:
        """Test getting dataloaders config."""
        result = load_dataloader_config()
        self.assertIs(type(result), list)
        self.assertIsNot(result, [])

    def test_load_transforms_config(self) -> None:
        """Test getting transforms config."""
        result = load_transforms_config()
        self.assertIs(type(result), list)
        self.assertIsNot(result, [])

    def test_load_metrics_help_lpot_params(self) -> None:
        """Test getting lpot metrics tooltips."""
        result = load_help_lpot_params("metrics")
        self.assertIs(type(result), dict)
        self.assertIsNot(result, {})

    def test_load_objectives_help_lpot_params(self) -> None:
        """Test getting lpot objectives tooltips."""
        result = load_help_lpot_params("objectives")
        self.assertIs(type(result), dict)
        self.assertIsNot(result, {})

    def test_load_strategies_help_lpot_params(self) -> None:
        """Test getting lpot strategies tooltips."""
        result = load_help_lpot_params("strategies")
        self.assertIs(type(result), dict)
        self.assertIsNot(result, {})

    def test_load_non_existing_help_lpot_params(self) -> None:
        """Test getting lpot strategies tooltips."""
        with self.assertRaises(FileNotFoundError):
            load_help_lpot_params("unknown_param")

    def verify_file_path(self) -> None:
        """Check if path can be accessed."""
        path = "/data"
        verify_file_path(path)

    def verify_root_path(self) -> None:
        """Check if path can be accessed."""
        path = "/"
        with self.assertRaises(AccessDeniedException):
            verify_file_path(path)

    def verify_hidden_path(self) -> None:
        """Check if path can be accessed."""
        path = "/home/user/.ssh/secret_key"
        with self.assertRaises(AccessDeniedException):
            verify_file_path(path)

    def verify_non_existing_path(self) -> None:
        """Check if path can be accessed."""
        path = "/some/non/existing/path"
        with self.assertRaises(NotFoundException):
            verify_file_path(path)

    def verify_restricted_path(self) -> None:
        """Check if path can be accessed."""
        path = "/usr"
        with self.assertRaises(NotFoundException):
            verify_file_path(path)

    def _assert_predefined_config_path(self, framework: str, domain: str) -> None:
        """Assert predefined config path."""
        result = get_predefined_config_path(framework, domain)
        expected = os.path.join(
            os.path.abspath(
                os.path.dirname(
                    inspect.getfile(get_predefined_config_path),
                ),
            ),
            "configs",
            "predefined_configs",
            f"{framework}",
            f"{domain}.yaml",
        )
        self.assertEqual(result, expected)
        self.assertEqual(os.path.isfile(result), True)

    def test_is_development_env(self) -> None:
        """Check if development env is activated."""
        os.environ.update({"LPOT_MODE": "development"})
        is_develop = is_development_env()
        self.assertTrue(is_develop)

    def test_is_production_env(self) -> None:
        """Check if production env is activated."""
        os.environ.update({"LPOT_MODE": "production"})
        is_develop = is_development_env()
        self.assertFalse(is_develop)

    def test_is_empty_lpot_mode_env(self) -> None:
        """Check if development env is activated."""
        if os.environ.get("LPOT_MODE", None) is not None:
            del os.environ["LPOT_MODE"]
        is_develop = is_development_env()
        self.assertFalse(is_develop)


if __name__ == "__main__":
    unittest.main()



#
#  -*- coding: utf-8 -*-
#

import unittest
import tensorflow as tf

from lpot.adaptor.tf_utils.graph_rewriter.generic.convert_leakyrelu import ConvertLeakyReluOptimizer
from tensorflow.python.framework import graph_util
from lpot.adaptor.tf_utils.util import disable_random

class TestConvertLeaklyRelu(unittest.TestCase):
    @disable_random()
    def test_convert_leakyrelu(self):
        tf.compat.v1.disable_eager_execution()
        x = tf.compat.v1.placeholder(tf.float32, [1, 3, 224, 224], name="input")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 3, 32],
                                    initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x, conv_weights, strides=[1, 1, 1, 1], 
                                    padding="SAME", data_format='NCHW')
        relu = tf.nn.relu(conv)
        mul = tf.math.multiply(relu, tf.constant([0.1]))
        maximum = tf.math.maximum(relu, mul)
        identity = tf.identity(maximum)
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[identity.name.split(':')[0]])
        post_graph = ConvertLeakyReluOptimizer(graph_def).do_transformation()
        converted = False
        for node in post_graph.node:
            if node.op == 'LeakyRelu':
                converted = True
        self.assertEqual(converted, True)
    
    @disable_random()
    def test_convert_leakyrelu_with_alpha_large_than_one(self):
        tf.compat.v1.disable_eager_execution()
        x = tf.compat.v1.placeholder(tf.float32, [1, 3, 224, 224], name="input")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 3, 32],
                                    initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x, conv_weights, strides=[1, 1, 1, 1], 
                                    padding="SAME", data_format='NCHW')
        relu = tf.nn.relu(conv)
        mul = tf.math.multiply(relu, tf.constant([3.1]))
        maximum = tf.math.maximum(relu, mul)
        identity = tf.identity(maximum)
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[identity.name.split(':')[0]])
        post_graph = ConvertLeakyReluOptimizer(graph_def).do_transformation()
        converted = False
        for node in post_graph.node:
            if node.op == 'LeakyRelu':
                converted = True
        self.assertEqual(converted, False)

    @disable_random()
    def test_convert_leakyrelu_with_invalid_maximum(self):
        tf.compat.v1.disable_eager_execution()
        x = tf.compat.v1.placeholder(tf.float32, [1, 3, 224, 224], name="input")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 3, 32],
                                    initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x, conv_weights, strides=[1, 1, 1, 1], 
                                    padding="SAME", data_format='NCHW')
        relu = tf.nn.relu(conv)
        mul = tf.math.multiply(relu, tf.constant([2.1]))

        maximum = tf.math.maximum(conv, mul)
        identity = tf.identity(maximum)
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[identity.name.split(':')[0]])
        post_graph = ConvertLeakyReluOptimizer(graph_def).do_transformation()
        converted = False
        for node in post_graph.node:
            if node.op == 'LeakyRelu':
                converted = True
        self.assertEqual(converted, False)

    @disable_random()
    def test_convert_leakyrelu_with_invalid_mul(self):
        tf.compat.v1.disable_eager_execution()
        x = tf.compat.v1.placeholder(tf.float32, [1, 3, 224, 224], name="input")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 3, 32],
                                    initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x, conv_weights, strides=[1, 1, 1, 1], 
                                    padding="SAME", data_format='NCHW')
        relu = tf.nn.relu(conv)
        const_identity = tf.identity(tf.constant([0.1]))
        mul = tf.math.multiply(relu, const_identity)

        maximum = tf.math.maximum(conv, mul)
        identity = tf.identity(maximum)
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[identity.name.split(':')[0]])
        post_graph = ConvertLeakyReluOptimizer(graph_def).do_transformation()
        converted = False
        for node in post_graph.node:
            if node.op == 'LeakyRelu':
                converted = True
        self.assertEqual(converted, False)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from lpot.utils.utility import LazyImport
from abc import abstractmethod
import collections
import numpy as np
from .sampler import IterableSampler, SequentialSampler, BatchSampler
from .fetcher import FETCHERS
from .default_dataloader import default_collate
from .default_dataloader import DefaultDataLoader
from ..datasets.bert_dataset import TensorflowBertDataset
from .base_dataloader import BaseDataLoader
import logging

tf = LazyImport('tensorflow')
lpot = LazyImport('lpot')

class TFDataDataLoader(BaseDataLoader):
    """In tensorflow1.x dataloader is coupled with the graph, but it also support feed_dict
       method to do session run, this dataloader is designed to satisfy the usage of feed dict
       in tf1.x. Although it's a general dataloader and can be used in MXNet and PyTorch.

    """

    def __init__(self, dataset, batch_size=1, last_batch='rollover'):

        self.dataset = dataset
        self.last_batch = last_batch
        self._batch_size = batch_size
        dataset = dataset.batch(batch_size)

    def batch(self, batch_size, last_batch='rollover'):
        drop_last = False if last_batch == 'rollover' else True
        self._batch_size = batch_size
        self.dataset = self.dataset.batch(batch_size, drop_last)

    def __iter__(self):
        return self._generate_dataloader(
            self.dataset,
            batch_size=self.batch_size,
            last_batch=self.last_batch,)

    def _generate_dataloader(self, dataset, batch_size=1, last_batch='rollover', \
                             collate_fn=None, sampler=None, batch_sampler=None, \
                             num_workers=None, pin_memory=None, shuffle=False):
        drop_last = False if last_batch == 'rollover' else True
        if shuffle:
            logging.warning('Shuffle is not supported yet in TFDataLoader, ' \
                            'ignoring shuffle keyword.')

        def check_dynamic_shape(element_spec):
            if isinstance(element_spec, collections.abc.Sequence):
                return any([check_dynamic_shape(ele) for ele in element_spec])
            elif isinstance(element_spec, tf.TensorSpec):
                return True if element_spec.shape.num_elements() is None else False
            else:
                raise ValueError('unrecognised element spec...')

        def squeeze_output(output):
            if isinstance(output, collections.abc.Sequence):
                return [squeeze_output(ele) for ele in output]
            elif isinstance(output, np.ndarray):
                return np.squeeze(output, axis=0)
            else:
                raise ValueError('not supported output format....')

        try_single_batch = check_dynamic_shape(dataset.element_spec)
        dataset = dataset.batch(1 if try_single_batch else batch_size, drop_last)
        if tf.executing_eagerly():
            for iter_tensors in dataset:
                yield [elem.numpy() for elem in iter_tensors]
        else:
            ds_iterator = tf.compat.v1.data.make_one_shot_iterator(dataset)
            iter_tensors = ds_iterator.get_next()
            data_config = tf.compat.v1.ConfigProto()
            data_config.use_per_session_threads = 1
            data_config.intra_op_parallelism_threads = 1
            data_config.inter_op_parallelism_threads = 16
            data_sess = tf.compat.v1.Session(config=data_config)
            # pylint: disable=no-name-in-module
            from tensorflow.python.framework.errors_impl import OutOfRangeError
            while True:
                try:
                    if not try_single_batch:
                        outputs = data_sess.run(iter_tensors)
                        yield outputs
                    else:
                        outputs = [squeeze_output(data_sess.run(iter_tensors)) \
                            for i in range(0, batch_size) ]
                        outputs = default_collate(outputs)
                        yield outputs
                except OutOfRangeError:
                    data_sess.close()
                    return

class TensorflowBertDataLoader(DefaultDataLoader):
    def _generate_dataloader(self, dataset, batch_size, last_batch, collate_fn,
                             sampler, batch_sampler, num_workers, pin_memory, shuffle):

        if shuffle:
            logging.warning('Shuffle is not supported yet in TensorflowBertDataLoader, ' \
                            'ignoring shuffle keyword.')
        def bert_collate_fn(batch):
            elem = batch[0]
            return elem
        drop_last = False if last_batch == 'rollover' else True
        sampler = self._generate_sampler(dataset)
        self.batch_sampler = BatchSampler(sampler, batch_size, drop_last)
        self.fetcher = FETCHERS[self.dataset_type](dataset, bert_collate_fn, drop_last)

        for batched_indices in self.batch_sampler:
            try:
                data = self.fetcher(batched_indices)
                yield (data[0], batch_size), data[1]
            except StopIteration:
                return

class TensorflowDataLoader(BaseDataLoader):
    """DataLoader for frameework Tensorflow, if it's a tf.data.Dataset we will directly use
       the dataloader in the other case will use DefaultDataLoader instead.

    """

    def _generate_dataloader(self, dataset, batch_size, last_batch, collate_fn, \
                sampler, batch_sampler, num_workers, pin_memory, shuffle):

        if shuffle:
            logging.warning('Shuffle is not supported yet in TensorflowDataLoader, ' \
                            'ignoring shuffle keyword.')
        if isinstance(dataset, tf.data.Dataset):
            return TFDataDataLoader(dataset, batch_size, last_batch=last_batch)
        elif isinstance(dataset, TensorflowBertDataset):
            return TensorflowBertDataLoader(dataset, batch_size, last_batch,
                        collate_fn, sampler, batch_sampler, num_workers,
                        pin_memory, shuffle)
        else:
            return DefaultDataLoader(dataset, batch_size, last_batch, collate_fn,
                                     sampler, batch_sampler, num_workers,
                                     pin_memory, shuffle)



"""Tests for lpot benchmark"""
import unittest
import os
import yaml
import numpy as np
import tensorflow as tf
import tempfile
import re
from lpot.adaptor.tf_utils.util import write_graph

def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: x
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
          performance:
            iteration: 20
            configs:
               cores_per_instance: 4
               num_of_instance: 2 
        tuning:
          accuracy_criterion:
            relative: 0.01
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()


def build_benchmark():
    seq = [
        "from argparse import ArgumentParser\n",
        "arg_parser = ArgumentParser(description='Parse args')\n",
        "arg_parser.add_argument('--input_model', dest='input_model', default='input_model', help='input model')\n",
        "args = arg_parser.parse_args()\n",

        "from lpot.data import DATASETS\n",
        "dataset = DATASETS('tensorflow')['dummy']((100, 256, 256, 1), label=True)\n",

        "from lpot.experimental import Benchmark, common\n",
        "benchmarker = Benchmark('fake_yaml.yaml')\n",
        "benchmarker.b_dataloader = common.DataLoader(dataset)\n",
        "benchmarker.model = args.input_model\n",
        "benchmarker()\n"
    ]

    with open('fake.py', "w", encoding="utf-8") as f:
        f.writelines(seq)

def build_fake_model():
    graph_path = tempfile.mkstemp(suffix='.pb')[1]
    try:
        graph = tf.Graph()
        graph_def = tf.GraphDef()
        with tf.Session(graph=graph) as sess:
            x = tf.placeholder(tf.float64, shape=(1, 256, 256, 1), name='x')
            y_1 = tf.constant(np.random.random((3, 3, 1, 1)), name='y_1')
            y_2 = tf.constant(np.random.random((3, 3, 1, 1)), name='y_2')
            conv1 = tf.nn.conv2d(input=x, filter=y_1, strides=[1, 1, 1, 1], \
                                 padding='VALID', name='conv1')
            op = tf.nn.conv2d(input=conv1, filter=y_2, strides=[1, 1, 1, 1], \
                              padding='VALID', name='op_to_store')

            sess.run(tf.global_variables_initializer())
            constant_graph = tf.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        write_graph(graph_def, graph_path)
    except:
        graph = tf.Graph()
        graph_def = tf.compat.v1.GraphDef()
        with tf.compat.v1.Session(graph=graph) as sess:
            x = tf.compat.v1.placeholder(tf.float64, shape=(1, 256, 256, 1), name='x')
            y_1 = tf.constant(np.random.random((3, 3, 1, 1)), name='y_1')
            y_2 = tf.constant(np.random.random((3, 3, 1, 1)), name='y_2')
            conv1 = tf.nn.conv2d(input=x, filters=y_1, strides=[1, 1, 1, 1], \
                                 padding='VALID', name='conv1')
            op = tf.nn.conv2d(input=conv1, filters=y_2, strides=[1, 1, 1, 1], \
                              padding='VALID', name='op_to_store')

            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = tf.compat.v1.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        write_graph(graph_def, graph_path)
    return graph_path


class TestObjective(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.graph_path = build_fake_model()
        build_fake_yaml()
        build_benchmark()

    @classmethod
    def tearDownClass(self):
        if os.path.exists('fake_yaml.yaml'):
            os.remove('fake_yaml.yaml')
        if os.path.exists('2_4_0.log'):
            os.remove('2_4_0.log')
        if os.path.exists('2_4_1.log'):
            os.remove('2_4_1.log')

    def test_benchmark(self):
        os.system("python fake.py --input_model={}".format(self.graph_path))
        for i in range(2):
            with open(f'2_4_{i}.log', "r") as f:
                for line in f:
                    throughput = re.search(r"Throughput:\s+(\d+(\.\d+)?) images/sec", line)
            self.assertIsNotNone(throughput)


if __name__ == "__main__":
    unittest.main()


import os
import shutil
import unittest
import copy
import onnx
import numpy as np
from onnx import helper, TensorProto, numpy_helper, onnx_pb
from onnxruntime.quantization.quant_utils import QuantizationMode
from lpot.adaptor.ox_utils.onnx_quantizer import ONNXQuantizer
import onnxruntime as ort


class TestAdaptorONNXRT(unittest.TestCase):

    qlinear_backend = QuantizationMode.QLinearOps
    integer_backend = QuantizationMode.IntegerOps
    q_config = {"weight":{'dtype': 3, 
                          'algorithm': 'minmax', 
                          'scheme':'sym', 
                          'granularity': 'per_tensor'},
                'activation':{'dtype': 2, 
                              'algorithm': 'minmax', 
                              'scheme':'asym', 
                              'granularity':'per_tensor'}
                }

    @classmethod
    def setUpClass(cls):
        os.makedirs('./onnxrt_test')

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree("./onnxrt_test", ignore_errors=True)

    def static_test(self, model, q_config, quantize_params, quantizable_op_types):
        quantizer = ONNXQuantizer(copy.deepcopy(model),
            q_config,
            self.qlinear_backend,
            True,
            quantize_params,
            quantizable_op_types)
        quantizer.quantize_model()
        assert quantizer.model.model

    def dynamic_test(self, model, q_config, quantize_params, quantizable_op_types):
        quantizer = ONNXQuantizer(copy.deepcopy(model),
            q_config,
            self.integer_backend,
            False,
            quantize_params,
            quantizable_op_types)
        quantizer.quantize_model()
        assert quantizer.model.model

    def test_conv(self):
        for op in ['Conv', 'FusedConv']:
            A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
            B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 3, 3])
            C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [1, 1, 5, 1])
            D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [1, 1, 5, 1])
            conv_node = onnx.helper.make_node(op, ['A', 'B', 'C'], ['D'], 
                                              name=op, 
                                              kernel_shape=[3, 3], 
                                              pads=[1, 1, 1, 1])
            graph = helper.make_graph([conv_node], 'test_graph_1', [A, B, C], [D])
            model = helper.make_model(graph)
            q_config = {op: self.q_config},
            quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                               "B": [np.float32(10.), np.uint8(0)],
                               "C": [np.float32(10.), np.uint8(0)],
                               "D": [np.float32(10.), np.uint8(0)]}       
            quantizable_op_types = [op]
            self.static_test(model, q_config, quantize_params, quantizable_op_types)

    def test_matmul(self):
        A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
        B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 5, 1])
        C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [1, 1, 5, 1])
        matmul_node = onnx.helper.make_node('MatMul', ['A', 'B'], ['C'], name='Matmul')
        graph = helper.make_graph([matmul_node], 'test_graph_1', [A, B], [C])
        model = helper.make_model(graph)
        q_config = {"Matmul": self.q_config}
        quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                           "B": [np.float32(10.), np.uint8(0)],
                           "C": [np.float32(10.), np.uint8(0)]}
        quantizable_op_types = ["Matmul"]
        self.static_test(model, q_config, quantize_params, quantizable_op_types)
        self.dynamic_test(model, q_config, quantize_params, quantizable_op_types)
        q_config = {"Matmul": {"weight":{'dtype': 3,
                               'algorithm': 'minmax',
                               'scheme':'sym',
                               'granularity': 'per_tensor'},
                     'activation':{'dtype': 3,
                                   'algorithm': 'minmax',
                                   'scheme':'asym',
                                   'granularity':'per_tensor'}}}
        quantize_params = {}
        self.dynamic_test(model, q_config, quantize_params, quantizable_op_types)

    def test_attention(self):
        A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
        B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 5, 5])
        C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [1, 1, 5, 5])
        D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [1, 1, 5, 5])
        node = onnx.helper.make_node('Attention', ['A', 'B', 'C'], ['D'], name='Attention')
        graph = helper.make_graph([node], 'test_graph_1', [A, B, C], [D])
        model = helper.make_model(graph)
        q_config = {"Attention": self.q_config}
        quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                           "B": [np.float32(10.), np.uint8(0)],
                           "C": [np.float32(10.), np.uint8(0)],
                           "D": [np.float32(10.), np.uint8(0)]}
        quantizable_op_types = ["Attention"]
        self.static_test(model, q_config, quantize_params, quantizable_op_types)
        self.dynamic_test(model, q_config, quantize_params, quantizable_op_types)

    def test_gather(self):
        a_value = np.random.randn(100, 4).astype(np.float32)
        A_init = helper.make_tensor('A', TensorProto.FLOAT, [100, 4], 
                                    a_value.reshape(400).tolist())
        b_value = np.random.randint(2, size=(1, 10)).astype(np.int32)
        B_init = helper.make_tensor('B', TensorProto.INT32, [1, 10],
                                    b_value.reshape(10).tolist())
        A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [100, 4])
        B = helper.make_tensor_value_info('B', TensorProto.INT32, [1, 10])
        C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [10, 4])
        node = onnx.helper.make_node('Gather', ['A', 'B'], ['C'], name='Gather')
        graph = helper.make_graph([node], 'test_graph_1', [A, B], [C], [A_init, B_init])
        model = helper.make_model(graph)
        q_config = {'Gather': {"weight":{'dtype': 3,
                                         'algorithm': 'minmax',
                                         'scheme':'sym',
                                         'granularity': 'per_tensor'},
                              'activation':{'dtype': 2,
                                         'algorithm': 'minmax',
                                         'scheme':'asym',
                                         'granularity':'per_tensor'}
                  }} 
        quantize_params = {"A": [np.float32(10.), np.uint8(0)]}
        quantizable_op_types = ["Gather"]
        self.static_test(model, q_config, quantize_params, quantizable_op_types)
        self.dynamic_test(model, q_config, quantize_params, quantizable_op_types)
        graph = helper.make_graph([node], 'test_graph_1', [A, B], [C])
        model = helper.make_model(graph)
        q_config = {'Gather': {"weight":{'dtype': 3,
                                         'algorithm': 'minmax',
                                         'scheme':'sym',
                                         'granularity': 'per_tensor'},
                              'activation':{'dtype': 2,
                                         'algorithm': 'minmax',
                                         'scheme':'asym',
                                         'granularity':'per_tensor'}
                  }}
        quantize_params = {}
        self.dynamic_test(model, q_config, quantize_params, quantizable_op_types)

    def test_split(self):
        a_value = np.random.randn(100, 4).astype(np.float32)
        A_init = helper.make_tensor('A', TensorProto.FLOAT, [100, 4],
                                    a_value.reshape(400).tolist())
        A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [100, 4])
        B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [50, 4])
        C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [50, 4])
        node = onnx.helper.make_node('Split', ['A'], ['B', 'C'], name='Split')
        graph = helper.make_graph([node], 'test_graph_1', [A], [B, C], [A_init])
        model = helper.make_model(graph)
        q_config = {'Split': {"weight":{'dtype': 3,
                                         'algorithm': 'minmax',
                                         'scheme':'sym',
                                         'granularity': 'per_tensor'},
                              'activation':{'dtype': 2,
                                         'algorithm': 'minmax',
                                         'scheme':'asym',
                                         'granularity':'per_tensor'}
                  }}
        quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                           "B": [np.float32(10.), np.uint8(0)],
                           "C": [np.float32(10.), np.uint8(0)]}
        quantizable_op_types = ["Split"]
        self.static_test(model, q_config, quantize_params, quantizable_op_types)

    def test_pad(self):
        b_value = np.array([0, 1, 1, 0, 1, 1]).astype(np.int64)
        B_init = helper.make_tensor('B', TensorProto.INT64, [6],
                                    b_value.reshape(6).tolist())
        B = helper.make_tensor_value_info('B', TensorProto.INT64, [6])
        C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [1, 7, 7])

        d_value = np.random.randn(1).astype(np.float32)
        D_init = helper.make_tensor('D', TensorProto.FLOAT, [1],
                                    d_value.reshape(1).tolist())
        D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [1])

        e_value = np.random.randn(1, 5, 5).astype(np.float32)
        E_init = helper.make_tensor('E', TensorProto.FLOAT, [1, 5, 5],
                                    e_value.reshape(25).tolist())
        E = helper.make_tensor_value_info('E', TensorProto.FLOAT, [1, 1, 5, 5])
        f_value = np.random.randn(1, 3, 3).astype(np.float32)
        F_init = helper.make_tensor('F', TensorProto.FLOAT, [1, 3, 3],
                                    f_value.reshape(9).tolist())
        F = helper.make_tensor_value_info('F', TensorProto.FLOAT, [1, 1, 3, 3])
        for mode in ["constant", "edge", "reflect", "constant_value", "constant_value_wo_init"]:
            conv_node = onnx.helper.make_node('Conv', ['E', 'F'], ['A'], 
                                              name='Conv', 
                                              kernel=[3, 3],
                                              padding=[1, 1, 1, 1])
            if mode == "constant_value":
                node = onnx.helper.make_node('Pad', ['A', 'B', 'D'], ['C'], name='Pad', mode="constant")
                graph = helper.make_graph([conv_node, node], 'test_graph_1', [E, F, B, D], [C], [E_init, F_init, B_init, D_init])
            elif mode == "constant_value_wo_init":
                node = onnx.helper.make_node('Pad', ['A', 'B', 'D'], ['C'], name='Pad', mode="constant")
                graph = helper.make_graph([conv_node, node], 'test_graph_1', [E, F, B, D], [C], [E_init, F_init, B_init])
            else:
                node = onnx.helper.make_node('Pad', ['A', 'B'], ['C'], name='Pad', mode=mode)
                graph = helper.make_graph([conv_node, node], 'test_graph_1', [E, F, B], [C], [E_init, F_init, B_init])
            model = helper.make_model(graph)
            q_config = {'Conv': self.q_config, 
                        'Pad': {'activation':{'dtype': 2,
                                             'algorithm': 'minmax',
                                             'scheme':'asym',
                                             'granularity':'per_tensor'}
                      }}
            quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                               "C": [np.float32(10.), np.uint8(0)],
                               "D": [np.float32(10.), np.uint8(0)],
                               "E": [np.float32(10.), np.uint8(0)],
                               "F": [np.float32(10.), np.uint8(0)]}
            quantizable_op_types = ["Conv", "Pad"]
            self.static_test(model, q_config, quantize_params, quantizable_op_types)

        node = onnx.helper.make_node('Pad', ['E', 'B', 'D'], ['C'], name='Pad', mode="constant")
        graph = helper.make_graph([node], 'test_graph_1', [E, B, D], [C], [E_init, B_init, D_init])
        model = helper.make_model(graph)
        q_config = {'Pad': {'activation':{'dtype': 2,
                                         'algorithm': 'minmax',
                                         'scheme':'asym',
                                         'granularity':'per_tensor'}
                  }}
        quantize_params = {"C": [np.float32(10.), np.uint8(0)],
                           "E": [np.float32(10.), np.uint8(0)]}
        quantizable_op_types = ["Pad"]
        self.static_test(model, q_config, quantize_params, quantizable_op_types)

    def test_binary(self):
        for op in ['Mul', 'Add']:
            A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 10])
            B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1])
            C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [1, 10])
            node = onnx.helper.make_node(op, ['A', 'B'], ['C'], name=op)
            graph = helper.make_graph([node], 'test_graph_1', [A, B], [C])
            model = helper.make_model(graph)
            q_config = {op: self.q_config}
            quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                               "B": [np.float32(10.), np.uint8(0)],
                               "C": [np.float32(10.), np.uint8(0)]}
            quantizable_op_types = [op]
            self.static_test(model, q_config, quantize_params, quantizable_op_types)
    
    def test_relu(self):
        A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
        B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 3, 3])
        D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [1, 1, 5, 5])
        conv_node = onnx.helper.make_node('Conv', ['A', 'B'], ['C'], 
                                          name='Conv', 
                                          kernel_shape=[3, 3], 
                                          pads=[1, 1, 1, 1])
        relu_node = onnx.helper.make_node('Relu', ['C'], ['D'], name='Relu')
        graph = helper.make_graph([conv_node, relu_node], 'test_graph_1', [A, B], [D])
        model = helper.make_model(graph, **{'opset_imports': [helper.make_opsetid('', 13)]})
        sess_options = ort.SessionOptions()
        sess_options.graph_optimization_level = ort.GraphOptimizationLevel.ORT_ENABLE_EXTENDED
        sess_options.optimized_model_filepath = "./onnxrt_test/optimized_model.onnx"  
        session = ort.InferenceSession(model.SerializeToString(), sess_options)
        model = onnx.load(sess_options.optimized_model_filepath)
 
        q_config = {"Conv": self.q_config, "Relu": self.q_config}
        quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                           "B": [np.float32(10.), np.uint8(0)],
                           "C": [np.float32(10.), np.uint8(0)],
                           "D": [np.float32(10.), np.uint8(0)]}
        quantizable_op_types = ["Conv", "Relu"]
        self.static_test(model, q_config, quantize_params, quantizable_op_types)
        
    def test_clip(self):
        A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
        B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 3, 3])
        D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [1, 1, 5, 5])
        conv_node = onnx.helper.make_node('Conv', ['A', 'B'], ['C'],
                                          name='Conv',
                                          kernel_shape=[3, 3],
                                          pads=[1, 1, 1, 1])
        clip_node = onnx.helper.make_node('Clip', ['C'], ['D'], name='Clip')
        graph = helper.make_graph([conv_node, clip_node], 'test_graph_1', [A, B], [D])
        model = helper.make_model(graph, **{'opset_imports': [helper.make_opsetid('', 13)]})

        sess_options = ort.SessionOptions()
        sess_options.graph_optimization_level = ort.GraphOptimizationLevel.ORT_ENABLE_EXTENDED
        sess_options.optimized_model_filepath = "./onnxrt_test/optimized_model.onnx"
        session = ort.InferenceSession(model.SerializeToString(), sess_options)
        model = onnx.load(sess_options.optimized_model_filepath)

        q_config = {"Conv": self.q_config, "Clip": self.q_config}
        quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                           "B": [np.float32(10.), np.uint8(0)],
                           "C": [np.float32(10.), np.uint8(0)],
                           "D": [np.float32(10.), np.uint8(0)]}
        quantizable_op_types = ["Conv", "Clip"]
        self.static_test(model, q_config, quantize_params, quantizable_op_types)

    def test_activation(self):
        for op in ["Relu", "LeakyRelu", "Sigmoid"]:
            B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 10])
            A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 10])
            node = onnx.helper.make_node(op, ['A'], ['B'], name=op)
            graph = helper.make_graph([node], 'test_graph_1', [A], [B])
            model = helper.make_model(graph)
            q_config = {op: self.q_config}
            quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                               "B": [np.float32(10.), np.uint8(0)]}
            quantizable_op_types = [op]
            self.static_test(model, q_config, quantize_params, quantizable_op_types)

            a_value = np.random.randn(1, 10).astype(np.float32)
            A_init = helper.make_tensor('A', TensorProto.FLOAT, [1, 10],
                                        a_value.reshape(10).tolist())
            graph = helper.make_graph([node], 'test_graph_1', [A], [B], [A_init])
            model = helper.make_model(graph)
            self.static_test(model, q_config, quantize_params, quantizable_op_types)

    def test_pooling(self):
        for op in ["MaxPool", "GlobalAveragePool"]:
            B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 5, 5])
            A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
            a_value = np.random.randn(1, 1, 5, 5).astype(np.float32)
            A_init = helper.make_tensor('A', TensorProto.FLOAT, [1, 1, 5, 5],
                                    a_value.reshape(25).tolist())
            node = onnx.helper.make_node(op, ['A'], ['B'], 
                                         name=op,
                                         kernel_shape=[3, 3],
                                         pads=[1, 1, 1, 1])
            graph = helper.make_graph([node], 'test_graph_1', [A], [B], [A_init]) 
            q_config = {op: self.q_config}
            quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                               "B": [np.float32(10.), np.uint8(0)]}
            quantizable_op_types = [op]
            for opset_version in [12, 13]:
                opset = onnx.OperatorSetIdProto()
                opset.version = opset_version
                model = helper.make_model(graph, opset_imports=[opset])
                self.static_test(model, q_config, quantize_params, quantizable_op_types)

            A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
            B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 3, 3])
            D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [1, 1, 5, 5])
            conv_node = onnx.helper.make_node('Conv', ['A', 'B'], ['C'],
                                              name='Conv',
                                              kernel_shape=[3, 3],
                                              pads=[1, 1, 1, 1])
            pool_node = onnx.helper.make_node(op, ['C'], ['D'], name=op)
            graph = helper.make_graph([conv_node, pool_node], 'test_graph_1', [A, B], [D])
            model = helper.make_model(graph)
 
            q_config = {"Conv": self.q_config, op: self.q_config}
            quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                               "B": [np.float32(10.), np.uint8(0)],
                               "C": [np.float32(10.), np.uint8(0)],
                               "D": [np.float32(10.), np.uint8(0)]}
            quantizable_op_types = ["Conv", op]
            self.static_test(model, q_config, quantize_params, quantizable_op_types)

    def test_exclude_node(self):
        A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
        B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 3, 3])
        D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [1, 1, 5, 5])
        conv_node = onnx.helper.make_node('Conv', ['A', 'B'], ['C'],
                                          name='Conv',
                                          kernel_shape=[3, 3],
                                          pads=[1, 1, 1, 1])
        pool_node = onnx.helper.make_node("MaxPool", ['C'], ['D'], name="MaxPool")
        graph = helper.make_graph([conv_node, pool_node], 'test_graph_1', [A, B], [D])
        model = helper.make_model(graph)

        q_config = {"Conv": self.q_config, "MaxPool": "fp32"}
        quantize_params = {"A": [np.float32(10.), np.uint8(0)],
                           "B": [np.float32(10.), np.uint8(0)],
                           "C": [np.float32(10.), np.uint8(0)],
                           "D": [np.float32(10.), np.uint8(0)]}
        quantizable_op_types = ["Conv", "MaxPool"]
        self.static_test(model, q_config, quantize_params, quantizable_op_types)

if __name__ == "__main__":
    unittest.main()



import os
import shutil
import unittest

import torch
import torchvision
import torch.nn as nn

from lpot.data import DATASETS
from lpot.experimental.data.dataloaders.pytorch_dataloader import PyTorchDataLoader

def build_fake_yaml():
    fake_yaml = """
    model:
      name: imagenet_prune
      framework: pytorch

    pruning:
      train:
        start_epoch: 0
        end_epoch: 4
        iteration: 10
        dataloader:
          batch_size: 30
          dataset:
            dummy:
              shape: [128, 3, 224, 224]
              label: True
        optimizer:
          SGD:
            learning_rate: 0.1
            momentum: 0.1
            nesterov: True
            weight_decay: 0.1
        criterion:
          CrossEntropyLoss:
            reduction: sum
      approach:
        weight_compression:
          initial_sparsity: 0.0
          target_sparsity: 0.97
          start_epoch: 0
          end_epoch: 4
          pruners:
            - !Pruner
                start_epoch: 1
                end_epoch: 3
                prune_type: basic_magnitude
                names: ['layer1.0.conv1.weight']

            - !Pruner
                target_sparsity: 0.6
                prune_type: gradient_sensitivity
                update_frequency: 2
                names: ['layer1.0.conv2.weight']
    evaluation:
      accuracy:
        metric:
          topk: 1
        dataloader:
          batch_size: 30
          dataset:
            dummy:
              shape: [128, 3, 224, 224]
              label: True
    """
    with open('fake.yaml', 'w', encoding="utf-8") as f:
        f.write(fake_yaml)


class TestPruning(unittest.TestCase):

    model = torchvision.models.resnet18()

    @classmethod
    def setUpClass(cls):
        build_fake_yaml()

    @classmethod
    def tearDownClass(cls):
        os.remove('fake.yaml')
        shutil.rmtree('./saved', ignore_errors=True)
        shutil.rmtree('runs', ignore_errors=True)

    def test_pruning_internal(self):
        from lpot.experimental import Pruning, common
        prune = Pruning('fake.yaml')

        prune.model = common.Model(self.model)
        _ = prune()

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Workdir test."""

import unittest
from copy import deepcopy
from unittest.mock import patch

from lpot.ux.utils.exceptions import InternalException, NotFoundException
from lpot.ux.utils.templates.workdir import Workdir

workloads_dict = {
    "active_workspace_path": "/tmp/lpot",
    "workloads": {
        "workload_1": {
            "id": "workload_1",
            "workload_path": "/tmp/lpot/workload_1",
        },
        "workload_2": {
            "id": "workload_2",
            "workload_path": "/tmp/lpot/workload_2",
        },
    },
}


def _fake_load_mock(self: Workdir) -> dict:
    self.workloads_data = deepcopy(workloads_dict)
    return self.workloads_data


def _fake_raise_exception_mock(*args: tuple, **kwargs: dict) -> None:
    raise Exception("Mocked exception.")


class TestDeleteWorkload(unittest.TestCase):
    """Workdir deletion tests."""

    def test_delete_workload_success(self) -> None:
        """Test workload deletion - success."""
        with patch("os.path.isfile") as fake_isfile, patch(
            "shutil.rmtree",
        ) as fake_rmtree, patch.object(Workdir, "load", autospec=True) as fake_load, patch.object(
            Workdir,
            "dump",
        ) as fake_dump:
            fake_isfile.return_value = True
            fake_rmtree.return_value = None
            fake_load.side_effect = _fake_load_mock
            fake_dump.return_value = None
            workdir = Workdir(workspace_path="/tmp/lpot")
            workdir.delete_workload("workload_1")
            self.assertNotIn("workload_1", workdir.workloads_data.get("workloads", {}))
            self.assertIn("workload_2", workdir.workloads_data.get("workloads", {}))

    def test_delete_workload_missing(self) -> None:
        """Test workload deletion - error due to missing file."""
        with patch("os.path.isfile") as fake_isfile, patch(
            "shutil.rmtree",
        ) as fake_rmtree, patch.object(Workdir, "load", autospec=True) as fake_load, patch.object(
            Workdir,
            "dump",
        ) as fake_dump, self.assertRaises(
            NotFoundException,
        ):
            fake_isfile.return_value = True
            fake_rmtree.return_value = None
            fake_load.side_effect = _fake_load_mock
            fake_dump.return_value = None
            workdir = Workdir(workspace_path="/tmp/lpot")
            workdir.delete_workload("workload_3")

    def test_delete_workload_failure(self) -> None:
        """Test workload deletion - error due to exception."""
        with patch("os.path.isfile") as fake_isfile, patch(
            "shutil.rmtree",
        ) as fake_rmtree, patch.object(Workdir, "load", autospec=True) as fake_load, patch.object(
            Workdir,
            "dump",
        ) as fake_dump, self.assertRaises(
            InternalException,
        ):
            fake_isfile.return_value = True
            fake_rmtree.side_effect = _fake_raise_exception_mock
            fake_load.side_effect = _fake_load_mock
            fake_dump.return_value = None
            workdir = Workdir(workspace_path="/tmp/lpot")
            workdir.delete_workload("workload_1")



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Execute tune."""

import json
import os
from typing import Any, Dict, Optional

from lpot.ux.components.optimization.factory import OptimizationFactory
from lpot.ux.components.optimization.optimization import Optimization
from lpot.ux.utils.exceptions import ClientErrorException
from lpot.ux.utils.executor import Executor
from lpot.ux.utils.logger import log
from lpot.ux.utils.parser import OptimizationParser
from lpot.ux.utils.templates.workdir import Workdir
from lpot.ux.utils.utils import _load_json_as_dict, get_size
from lpot.ux.web.communication import MessageQueue

mq = MessageQueue()


def execute_optimization(data: Dict[str, Any]) -> dict:
    """Get configuration."""
    from lpot.ux.utils.workload.workload import Workload

    if not str(data.get("id", "")):
        message = "Missing request id."
        mq.post_error(
            "optimization_finish",
            {"message": message, "code": 404},
        )
        raise Exception(message)

    request_id: str = data["id"]

    workdir: Optional[Workdir] = None

    try:
        workdir = Workdir(request_id=request_id, overwrite=False)
        workload_path: str = workdir.workload_path
        workload_data = _load_json_as_dict(
            os.path.join(workload_path, "workload.json"),
        )
        workload = Workload(workload_data)
        optimization: Optimization = OptimizationFactory.get_optimization(
            workload,
            workdir.template_path,
        )
        send_data = {
            "message": "started",
            "id": request_id,
            "size_input_model": get_size(optimization.input_graph),
        }
        workdir.clean_logs()
        workdir.update_data(
            request_id=request_id,
            model_path=optimization.input_graph,
            input_precision=optimization.input_precision,
            model_output_path=optimization.output_graph,
            output_precision=optimization.output_precision,
            status="wip",
        )

        executor = Executor(
            workspace_path=workload_path,
            subject="optimization",
            data=send_data,
            log_name="output",
        )

        proc = executor.call(
            optimization.command,
        )
        optimization_time = executor.process_duration
        if optimization_time:
            optimization_time = round(optimization_time, 2)
        log.debug(f"Elapsed time: {optimization_time}")
        logs = [os.path.join(workload_path, "output.txt")]
        parser = OptimizationParser(logs)
        if proc.is_ok:
            response_data = parser.process()

            optimized_model_path = response_data.get(
                "path_optimized_model",
                optimization.output_graph,
            )
            optimization.output_graph = optimized_model_path

            if isinstance(response_data, dict):
                response_data["id"] = request_id
                response_data["optimization_time"] = optimization_time
                response_data["size_optimized_model"] = get_size(optimized_model_path)
                response_data["model_output_path"] = optimized_model_path
                response_data["size_input_model"] = get_size(optimization.input_graph)
                response_data["is_custom_dataloader"] = bool(workdir.template_path)

                workdir.update_data(
                    request_id=request_id,
                    model_path=optimization.input_graph,
                    model_output_path=optimized_model_path,
                    metric=response_data,
                    status="success",
                    execution_details={"optimization": optimization.serialize()},
                    input_precision=optimization.input_precision,
                    output_precision=optimization.output_precision,
                )
                response_data["execution_details"] = {"optimization": optimization.serialize()}

            log.debug(f"Parsed data is {json.dumps(response_data)}")
            mq.post_success("optimization_finish", response_data)
            return response_data
        else:
            log.debug("FAIL")
            workdir.update_data(
                request_id=request_id,
                model_path=optimization.input_graph,
                input_precision=optimization.input_precision,
                output_precision=optimization.output_precision,
                status="error",
            )
            raise ClientErrorException("Optimization failed during execution.")
    except Exception as err:
        mq.post_failure("optimization_finish", {"message": str(err), "id": request_id})
        if workdir is not None and workdir.get_workload_data(request_id).get("status") != "error":
            workdir.update_data(
                request_id=request_id,
                status="error",
            )
        raise



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""UX server HW info module."""

import platform
import re
import subprocess
from typing import Union

import cpuinfo
import psutil

from lpot.ux.utils.logger import log
from lpot.ux.utils.utils import determine_ip


class HWInfo:
    """Class responsible for gathering information about platform hardware."""

    def __init__(self) -> None:
        """Initialize HW Info class and gather information platform hardware."""
        cpu_info = cpuinfo.get_cpu_info()
        self.sockets: int = get_number_of_sockets()
        self.cores: int = psutil.cpu_count(logical=False)
        self.cores_per_socket: int = int(psutil.cpu_count(logical=False) / self.sockets)
        self.threads_per_socket: int = int(psutil.cpu_count(logical=True) / self.sockets)
        self.total_memory: str = f"{psutil.virtual_memory().total / (1024 ** 3):.3f}GB"
        self.system: str = get_distribution()
        self.ip = determine_ip()
        if cpu_info.get("brand"):
            self.platform = cpu_info.get("brand")
        elif cpu_info.get("brand_raw"):
            self.platform = cpu_info.get("brand_raw")
        else:
            self.platform = ""
        self.hyperthreading_enabled: bool = psutil.cpu_count(logical=False) != psutil.cpu_count(
            logical=True,
        )
        self.turboboost_enabled = is_turbo_boost_enabled()
        self.bios_version = get_bios_version()
        self.kernel: str = get_kernel_version()
        try:
            min_cpu_freq = int(psutil.cpu_freq(percpu=False).min)
            max_cpu_freq = int(psutil.cpu_freq(percpu=False).max)
        except Exception:
            log.warning("Cannot collect cpu frequency information.")
            min_cpu_freq = 0
            max_cpu_freq = 0
        finally:
            if min_cpu_freq == 0:
                self.min_cpu_freq = "n/a"
            else:
                self.min_cpu_freq = f"{min_cpu_freq}Hz"
            if max_cpu_freq == 0:
                self.max_cpu_freq = "n/a"
            else:
                self.max_cpu_freq = f"{max_cpu_freq}Hz"


def get_number_of_sockets() -> int:
    """Get number of sockets in platform."""
    cmd = "lscpu | grep 'Socket(s)' | cut -d ':' -f 2"

    proc = subprocess.Popen(
        args=cmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        universal_newlines=False,
    )
    proc.wait()
    if proc.stdout:
        for line in proc.stdout:
            return int(line.decode("utf-8", errors="ignore").strip())
    return 0


def get_distribution() -> str:
    """
    Return system distibution.

    :return: distribution name
    :rtype: str
    """
    if psutil.WINDOWS:
        return f"{platform.system()} {platform.release()}"
    elif psutil.LINUX:
        try:
            return " ".join(platform.dist())
        except AttributeError:
            return f"{platform.system()} {platform.release()}"
    else:
        return platform.platform()


def get_bios_version() -> str:
    """Return bios version."""
    if psutil.LINUX:
        try:
            cmd = "cat /sys/class/dmi/id/bios_version"

            proc = subprocess.Popen(
                args=cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                universal_newlines=False,
            )
            proc.wait()
            if proc.stdout and proc.returncode == 0:
                for line in proc.stdout:
                    bios_version = line.decode("utf-8", errors="ignore").strip()
            else:
                bios_version = "n/a"

            cmd = "grep microcode  /proc/cpuinfo -m1"
            proc = subprocess.Popen(
                args=cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                universal_newlines=False,
            )
            proc.wait()
            if proc.stdout and proc.returncode == 0:
                for line in proc.stdout:
                    microcode_raw = line.decode("utf-8", errors="ignore").strip()
                    result = re.search(r"microcode.*: (.*)", microcode_raw)
                    if result:
                        microcode = result.group(1)
            else:
                microcode = "n/a"
            return f"BIOS {bios_version} Microcode {microcode}"
        except Exception:
            return "n/a"
    return "n/a"


def is_turbo_boost_enabled() -> Union[str, bool]:
    """
    Check if turbo boost is enabled.

    Return True if enabled, False if disabled, None if cannot collect info.
    """
    if psutil.LINUX:
        try:
            cmd = "cat /sys/devices/system/cpu/intel_pstate/no_turbo"
            proc = subprocess.Popen(
                args=cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                universal_newlines=False,
            )
            proc.wait()
            if proc.stdout:
                for line in proc.stdout:
                    raw_status = int(line.decode("utf-8", errors="ignore").strip())
                    if raw_status == 0:
                        return True
                    elif raw_status == 1:
                        return False
                    else:
                        return "n/a"
        except Exception:
            return "n/a"
    return "n/a"


def get_kernel_version() -> str:
    """Return kernel version."""
    if psutil.LINUX:
        return platform.release()
    else:
        return platform.platform()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Optimization class factory."""

from typing import Optional

from lpot.ux.components.optimization import Optimizations
from lpot.ux.components.optimization.graph_optimizer.graph_optimization import GraphOptimization
from lpot.ux.components.optimization.optimization import Optimization
from lpot.ux.components.optimization.tune.tuning import Tuning
from lpot.ux.utils.exceptions import InternalException
from lpot.ux.utils.logger import log
from lpot.ux.utils.workload.workload import Workload


class OptimizationFactory:
    """Optimization factory."""

    @staticmethod
    def get_optimization(workload: Workload, template_path: Optional[str] = None) -> Optimization:
        """Get optimization for specified workload."""
        optimization_map = {
            Optimizations.TUNING: Tuning,
            Optimizations.GRAPH: GraphOptimization,
        }
        optimization = optimization_map.get(workload.mode, None)
        if optimization is None:
            raise InternalException(f"Could not find optimization class for {workload.mode}")
        log.debug(f"Initializing {optimization.__name__} class.")
        return optimization(workload, template_path)



"""Tests for coco_tools. """
import unittest
import numpy as np
from lpot.experimental.metric.coco_tools import *

class TestCOCO(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        groundtruth_annotations_list = [
            {
                'id': 1,
                'image_id': 1,
                'category_id': 1,
                'bbox': [387.99,97.43,84.99,81.29],
                'area': 2991.9213,
                'iscrowd': 0,
                'segmentation':[
                    [387.99,176.5,398.34,164.68,405.733,156.55,412.38,141.77,
                     419.77,136.6,424.94,125.51,432.33,116.64,434.55,102.6,
                     436.77,97.43,441.944,102.6,453.76,101.12,459.68,109.99,
                     457.46,115.9,463.37,124.03,470.76,128.47,472.98,137.34,
                     465.559,143.25,447.11,137.34,444.9,142.51,442.68,156.55,
                     444.9,163.2,446.37,176.5,444.9,178.72]
                 ]
            }
        ]
        image_list = [{'id': 1}]
        category_list = [{'id': 0, 'name': 'person'},
                         {'id': 1, 'name': 'cat'},
                         {'id': 2, 'name': 'dog'}]
        cls.groundtruth_dict = {
            'annotations': groundtruth_annotations_list,
            'images': image_list,
            'categories': category_list
        }
        cls.detections_list = [
            {
                'image_id': 1,
                'category_id': 1,
                'bbox': [387.99,97.43,84.99,81.29],
                'score': .8,
                'segmentation':[
                    [387.99,176.5,398.34,164.68,405.733,156.55,412.38,141.77,
                     419.77,136.6,424.94,125.51,432.33,116.64,434.55,102.6,
                     436.77,97.43,441.944,102.6,453.76,101.12,459.68,109.99,
                     457.46,115.9,463.37,124.03,470.76,128.47,472.98,137.34,
                     465.559,143.25,447.11,137.34,444.9,142.51,442.68,156.55,
                     444.9,163.2,446.37,176.5,444.9,178.72]
                 ]
 
            },
        ]

    def testCOCOWrapper(self):
        with self.assertRaises(ValueError):
            wrap = COCOWrapper(None, 'test')

        wrap = COCOWrapper(TestCOCO.groundtruth_dict)
        with self.assertRaises(ValueError):
            wrap.LoadAnnotations(TestCOCO.groundtruth_dict)
        wrong_detection = {
            'image_id': 'test',
            'category_id': 1,
            'bbox': [100., 100., 100., 100.],
            'score': .8
        }
        with self.assertRaises(ValueError):
            wrap.LoadAnnotations(wrong_detection)
        wrong_detection = [
            {
            'image_id': 'test',
            'category_id': 1,
            'bbox': [100., 100., 100., 100.],
            'score': .8
            }
        ]
        with self.assertRaises(ValueError):
            wrap.LoadAnnotations(wrong_detection)
 
        groundtruth = COCOWrapper(TestCOCO.groundtruth_dict)
        detections = groundtruth.LoadAnnotations(TestCOCO.detections_list)
        evaluator = COCOEvalWrapper(groundtruth, detections)
        self.assertEqual(evaluator.GetCategory(1)['name'], 'cat')
        self.assertTrue(not evaluator.GetAgnosticMode())
        self.assertEqual(evaluator.GetCategoryIdList(), [0, 1, 2])
        evaluator = COCOEvalWrapper(groundtruth, detections, agnostic_mode=True)
        self.assertTrue(evaluator.GetAgnosticMode())
        summary_metrics, _ = evaluator.ComputeMetrics()
        self.assertAlmostEqual(1.0, summary_metrics['Precision/mAP'])
        with self.assertRaises(ValueError):
            summary_metrics, _ = evaluator.ComputeMetrics(True, True)

        iou_thrs = '0.5:0.05:0.95'
        map_points = 101
        evaluator = COCOEvalWrapper(groundtruth, detections, iou_thrs=iou_thrs, map_points=map_points)
        evaluator.evaluate()
        evaluator.accumulate()
        self.assertEqual(evaluator.eval['counts'], [10, 101, 3, 4, 3])

        iou_thrs = 0.5
        map_points = 11
        evaluator = COCOEvalWrapper(groundtruth, detections, iou_thrs=iou_thrs, map_points=map_points)
        evaluator.evaluate()
        evaluator.accumulate()
        self.assertEqual(evaluator.eval['counts'], [1, 11, 3, 4, 3])

        iou_thrs = 0.5
        map_points = 0
        evaluator = COCOEvalWrapper(groundtruth, detections, iou_thrs=iou_thrs, map_points=map_points)
        evaluator.evaluate()
        evaluator.accumulate()
        self.assertEqual(evaluator.eval['counts'], [1, 1, 3, 4, 3])



    def testExportSingleImageDetectionBoxesToCoco(self):
        with self.assertRaises(ValueError):
            ExportSingleImageDetectionBoxesToCoco(None, None, None, 
                    np.array([0]), np.array([[0,0]]))
        with self.assertRaises(ValueError):
            ExportSingleImageDetectionBoxesToCoco(None, None, np.array([0,0]), 
                    np.array([0]), np.array([0]))
        with self.assertRaises(ValueError):
            ExportSingleImageDetectionBoxesToCoco(None, None, np.array([[0,0]]), 
                    np.array([0]), np.array([0]))
    
    def testExportSingleImageGroundtruthToCoco(self):
        with self.assertRaises(ValueError):
            ExportSingleImageGroundtruthToCoco(None, None, None, 
                    np.array([0,0]), np.array([0]))
        with self.assertRaises(ValueError):
            ExportSingleImageGroundtruthToCoco(None, None, None, 
                    np.array([[0,0]]), np.array([0]))
        with self.assertRaises(ValueError):
            ExportSingleImageGroundtruthToCoco(None, None, None, 
                np.array([[1,1,5,5]]), np.array([1]), np.array([[[1]]]), np.array([[1,0]]))
        ExportSingleImageGroundtruthToCoco(1, 2, [0,1,2], np.array([[1,1,5,5]]), 
                np.array([1]), np.array([[[1]]], dtype=np.uint8), np.array([1,0]))


    def testExportSingleImageDetectionMasksToCoco(self):
        with self.assertRaises(ValueError):
            ExportSingleImageDetectionMasksToCoco(None, None, None, 
                    np.array([0]), np.array([[0,0]]))
        with self.assertRaises(ValueError):
            ExportSingleImageDetectionMasksToCoco(None, None, np.array([0,0]), 
                    np.array([0]), np.array([0]))
        mask=[
            [387.99,176.5,398.34,164.68,405.733,156.55,412.38,141.77,
             419.77,136.6,424.94,125.51,432.33,116.64,434.55,102.6,
             436.77,97.43,441.944,102.6,453.76,101.12,459.68,109.99,
             457.46,115.9,463.37,124.03,470.76,128.47,472.98,137.34,
             465.559,143.25,447.11,137.34,444.9,142.51,442.68,156.55,
             444.9,163.2,446.37,176.5,444.9,178.72]
         ]
 
        result = ExportSingleImageDetectionMasksToCoco(
                    1, [0,1,2], mask, np.array([0.8]), np.array([1]))
        self.assertEqual(len(result), 1)

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""HWInfo test."""

import unittest
from unittest.mock import MagicMock, patch

from lpot.ux.utils.hw_info import HWInfo


class TestHWInfo(unittest.TestCase):
    """HWInfo tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Hardware Info tests constructor."""
        super().__init__(*args, **kwargs)

    def test_HWInfo_type(self) -> None:
        """Test if hardware info collects."""
        hw_info_expected_types = {
            "bios_version": str,
            "cores": int,
            "cores_per_socket": int,
            "hyperthreading_enabled": bool,
            "ip": str,
            "kernel": str,
            "max_cpu_freq": str,
            "min_cpu_freq": str,
            "platform": str,
            "sockets": int,
            "system": str,
            "threads_per_socket": int,
            "total_memory": str,
            "turboboost_enabled": (bool, str),
        }
        hw_info = HWInfo()
        hw_info_dict = vars(hw_info)
        self.assertSetEqual(set(hw_info_dict.keys()), set(hw_info_expected_types.keys()))
        for key, value in hw_info_dict.items():
            if key in hw_info_expected_types:
                expected_type = hw_info_expected_types[key]
                if isinstance(expected_type, (type, tuple)):
                    if isinstance(value, expected_type):
                        continue
            raise Exception(
                f"Wrong type for key: {key} value: {value}. "
                f"Recived {type(value)}, expected {hw_info_expected_types.get(key)}.",
            )

    @patch("psutil.cpu_count")
    def test_cores_num(self, mock_cpu_count: MagicMock) -> None:
        """Test if hw info uses psutil cpu_count to get number of cores."""
        mock_cpu_count.return_value = 8

        hw_info = HWInfo()
        self.assertEqual(hw_info.cores, 8)

    @patch("subprocess.Popen")
    def test_sockets_num(self, mock_subprocess: MagicMock) -> None:
        """Test getting number of sockets."""
        mock_subprocess.return_value.stdout = [b"           4\n"]

        hw_info = HWInfo()
        self.assertEqual(hw_info.sockets, 4)

    @patch("platform.release")
    @patch("platform.system")
    @patch("psutil.LINUX", False)
    @patch("psutil.WINDOWS", True)
    def test_get_windows_distribution(
        self,
        mock_platform_system: MagicMock,
        mock_platform_release: MagicMock,
    ) -> None:
        """Test getting windows system distribution."""
        mock_platform_system.return_value = "Windows"
        mock_platform_release.return_value = "10"

        hw_info = HWInfo()
        self.assertEqual(hw_info.system, "Windows 10")

    @patch("platform.dist", create=True)
    @patch("psutil.LINUX", True)
    @patch("psutil.WINDOWS", False)
    def test_get_linux_distribution(
        self,
        mock_platform_dist: MagicMock,
    ) -> None:
        """Test getting linux system distribution."""
        mock_platform_dist.return_value = (
            "DistroName",
            "DistroVerID",
            "DistroVerCodename",
        )

        hw_info = HWInfo()
        self.assertEqual(hw_info.system, "DistroName DistroVerID DistroVerCodename")

    @patch("platform.release")
    @patch("platform.system")
    @patch("platform.dist", create=True)
    @patch("psutil.LINUX", True)
    @patch("psutil.WINDOWS", False)
    def test_get_linux_distribution_without_dist(
        self,
        mock_platform_dist: MagicMock,
        mock_platform_system: MagicMock,
        mock_platform_release: MagicMock,
    ) -> None:
        """Test getting linux system distribution."""
        mock_platform_dist.configure_mock(side_effect=AttributeError)
        mock_platform_system.return_value = "Linux"
        mock_platform_release.return_value = "kernel_ver-88-generic"

        hw_info = HWInfo()
        self.assertEqual(hw_info.system, "Linux kernel_ver-88-generic")

    @patch("platform.platform")
    @patch("psutil.LINUX", False)
    @patch("psutil.WINDOWS", False)
    def test_get_unknown_os_distribution(
        self,
        mock_platform_platform: MagicMock,
    ) -> None:
        """Test getting unknown system distribution."""
        mock_platform_platform.return_value = "SystemName-Version-Arch"

        hw_info = HWInfo()
        self.assertEqual(hw_info.system, "SystemName-Version-Arch")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Model config test."""

import unittest
from typing import Any, List

from lpot.ux.utils.workload.model import Model, parse_nodes


class TestModelConfig(unittest.TestCase):
    """Model config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Model config constructor."""
        super().__init__(*args, **kwargs)

    def test_parse_none_nodes(self) -> None:
        """Test parsing None to empty list."""
        nodes = None
        result = parse_nodes(nodes)
        self.assertEqual(result, [])

    def test_parse_string_nodes(self) -> None:
        """Test parsing string to empty list."""
        nodes = "input"
        result = parse_nodes(nodes)
        self.assertEqual(result, ["input"])

    def test_parse_nodes_empty_list(self) -> None:
        """Test parsing list to list."""
        nodes: List[Any] = []
        result = parse_nodes(nodes)
        self.assertEqual(result, [])

    def test_parse_nodes_list(self) -> None:
        """Test parsing nodes list to list."""
        nodes = ["input_1", "input_2"]
        result = parse_nodes(nodes)
        self.assertEqual(result, ["input_1", "input_2"])

    def test_string_single_nodes_constructor(self) -> None:
        """Test if path is correctly recognized as hidden."""
        model_config = Model(
            {
                "name": "model_name",
                "framework": "framework_name",
                "inputs": "input",
                "outputs": "predict",
            },
        )
        self.assertEqual(model_config.inputs, "input")
        self.assertEqual(model_config.outputs, "predict")

    def test_string_multiple_nodes_constructor(self) -> None:
        """Test if path is correctly recognized as hidden."""
        model_config = Model(
            {
                "name": "model_name",
                "framework": "framework_name",
                "inputs": "input, another_input",
                "outputs": "predict, softmax",
            },
        )
        self.assertEqual(model_config.inputs, "input, another_input")
        self.assertEqual(model_config.outputs, "predict, softmax")

    def test_string_missing_nodes_constructor(self) -> None:
        """Test if path is correctly recognized as hidden."""
        model_config = Model(
            {
                "name": "model_name",
                "framework": "framework_name",
            },
        )
        self.assertEqual(model_config.inputs, [])
        self.assertEqual(model_config.outputs, [])

    def test_inputs_setter(self) -> None:
        """Test model config inputs setter."""
        model_config = Model(
            {
                "name": "model_name",
                "framework": "framework_name",
            },
        )
        self.assertEqual(model_config.inputs, [])
        model_config.inputs = "input"
        self.assertEqual(model_config.inputs, ["input"])

    def test_outputs_setter(self) -> None:
        """Test model config outputs setter."""
        model_config = Model(
            {
                "name": "model_name",
                "framework": "framework_name",
            },
        )
        self.assertEqual(model_config.outputs, [])
        model_config.outputs = ["softmax", "predict"]
        self.assertEqual(model_config.outputs, ["softmax", "predict"])

    def test_model_without_nodes_serialization(self) -> None:
        """Test model config serialization."""
        model_config = Model(
            {
                "name": "model_name",
                "framework": "framework_name",
            },
        )
        result = model_config.serialize()
        expected = {
            "name": "model_name",
            "framework": "framework_name",
        }
        self.assertDictEqual(result, expected)

    def test_model_single_node_serialization(self) -> None:
        """Test model config serialization."""
        model_config = Model(
            {
                "name": "model_name",
                "framework": "framework_name",
                "inputs": "input",
                "outputs": "predict",
            },
        )
        result = model_config.serialize()
        expected = {
            "name": "model_name",
            "framework": "framework_name",
            "inputs": "input",
            "outputs": "predict",
        }
        self.assertDictEqual(result, expected)

    def test_model_multiple_nodes_serialization(self) -> None:
        """Test model config serialization."""
        model_config = Model(
            {
                "name": "model_name",
                "framework": "framework_name",
                "inputs": ["input"],
                "outputs": ["predict"],
            },
        )
        result = model_config.serialize()
        expected = {
            "name": "model_name",
            "framework": "framework_name",
            "inputs": "input",
            "outputs": "predict",
        }
        self.assertDictEqual(result, expected)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import onnx
from .base_operator import QuantOperatorBase
from onnxruntime.quantization.quant_utils import attribute_to_kwarg, ms_domain, \
                                                 QuantizedValueType
from onnx import onnx_pb as onnx_proto
from lpot.adaptor.ox_utils.util import QuantizedValue

class QLinearBinaryOp(QuantOperatorBase):
    def __init__(self, onnx_quantizer, onnx_node):
        super().__init__(onnx_quantizer, onnx_node)

    def quantize(self):
        node = self.node

        data_found, output_scale_name, output_zp_name, _, _ = \
            self.quantizer._get_quantization_params(node.output[0])
        if (not data_found):  # only try to quantize when given quantization parameters for it
            return super().quantize()

        (quantized_input_names, zero_point_names, scale_names, nodes) = \
            self.quantizer.quantize_inputs(node, [0, 1], initializer_use_weight_qType=False)

        qlinear_binary_math_output = node.output[0] + "_quantized"
        qlinear_binary_math_name = node.name + "_quant" if node.name != "" else ""

        kwargs = {}
        for attribute in node.attribute:
            kwargs.update(attribute_to_kwarg(attribute))
        kwargs["domain"] = ms_domain

        qlinear_binary_math_inputs = []
        # Input 0
        qlinear_binary_math_inputs.append(quantized_input_names[0])
        qlinear_binary_math_inputs.append(scale_names[0])
        qlinear_binary_math_inputs.append(zero_point_names[0])
        # Input 1
        qlinear_binary_math_inputs.append(quantized_input_names[1])
        qlinear_binary_math_inputs.append(scale_names[1])
        qlinear_binary_math_inputs.append(zero_point_names[1])

        # Output
        qlinear_binary_math_inputs.append(output_scale_name)
        qlinear_binary_math_inputs.append(output_zp_name)

        qlinear_binary_math_node = onnx.helper.make_node("QLinear" + node.op_type, 
                                                         qlinear_binary_math_inputs,
                                                         [qlinear_binary_math_output], 
                                                         qlinear_binary_math_name,
                                                         **kwargs)
        nodes.append(qlinear_binary_math_node)

        # Create an entry for this quantized value
        q_output = QuantizedValue(node.output[0], qlinear_binary_math_output, 
                                  output_scale_name, output_zp_name,
                                  QuantizedValueType.Input)
        self.quantizer.quantized_value_map[node.output[0]] = q_output

        self.quantizer.new_nodes += nodes



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Benchmark class."""

import os
import re
from typing import Any, Dict

from lpot.ux.components.benchmark import Benchmarks
from lpot.ux.components.benchmark.benchmark_model import benchmark_model
from lpot.ux.utils.exceptions import ClientErrorException
from lpot.ux.utils.hw_info import HWInfo
from lpot.ux.utils.workload.workload import Workload


class Benchmark:
    """Benchmark class."""

    def __init__(
        self,
        workload: Workload,
        model_path: str,
        mode: str,
        precision: str,
    ) -> None:
        """Initialize Benchmark class."""
        hw_info = HWInfo()
        self.instances: int = 1
        if mode == Benchmarks.PERF:
            self.instances = workload.config.get_performance_num_of_instance() or 1

        self.cores_per_instance: int = hw_info.cores // self.instances
        if mode == Benchmarks.PERF:
            self.cores_per_instance = (
                workload.config.get_performance_cores_per_instance()
                or hw_info.cores // self.instances
            )

        self.model_path = model_path
        self.precision = precision
        self.mode = mode
        self.batch_size = 1
        self.framework = workload.framework
        if (
            workload.config
            and workload.config.evaluation
            and workload.config.evaluation.performance
            and workload.config.evaluation.performance.dataloader
            and workload.config.evaluation.performance.dataloader.batch_size
        ):
            self.batch_size = workload.config.evaluation.performance.dataloader.batch_size

        if not os.path.exists(self.model_path):
            raise ClientErrorException(
                f"Could not find model at specified path: {self.model_path}.",
            )

        self.config_path = workload.config_path
        self.benchmark_script = os.path.join(
            os.path.dirname(__file__),
            "benchmark_model.py",
        )

        self.command = [
            "python",
            self.benchmark_script,
            "--config",
            self.config_path,
            "--input-graph",
            self.model_path,
            "--mode",
            self.mode,
            "--framework",
            self.framework,
        ]

    def execute(self) -> None:
        """Execute benchmark and collect results."""
        benchmark_model(
            input_graph=self.model_path,
            config=self.config_path,
            benchmark_mode=self.mode,
            framework=self.framework,
        )

    def serialize(self) -> Dict[str, Any]:
        """Serialize Benchmark to dict."""
        result = {}
        for key, value in self.__dict__.items():
            variable_name = re.sub(r"^_", "", key)
            if variable_name == "command":
                result[variable_name] = " ".join(value)
            else:
                result[variable_name] = value
        return result



import unittest
import numpy as np
import tensorflow as tf
from tensorflow.core.framework import attr_value_pb2
from tensorflow.core.framework import graph_pb2
from tensorflow.core.framework import node_def_pb2
from tensorflow.python.framework import tensor_util
from lpot.adaptor.tf_utils.graph_rewriter.generic.fold_constant import GraphFoldConstantOptimizer


class TestFoldConstant(unittest.TestCase):
    x_node = node_def_pb2.NodeDef()
    x_node.name = "placeholder"
    x_node.op = "Placeholder"

    input0_node = node_def_pb2.NodeDef()
    input0_node.name = "input0"
    input0_node.op = "Const"
    input0_value = np.float32(np.abs(np.random.randn(4, 3, 2)))
    input0_node.attr["value"].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            input0_value, input0_value.dtype.type, input0_value.shape)))

    input1_node = node_def_pb2.NodeDef()
    input1_node.name = "input1"
    input1_node.op = "Const"
    input1_value = np.float32(np.abs(np.random.randn(4, 1, 1)))
    input1_node.attr["value"].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            input1_value, input1_value.dtype.type, input1_value.shape)))

    add_node = node_def_pb2.NodeDef()
    add_node.op = "Add"
    add_node.name = "add"
    add_node.input.extend([input0_node.name, input1_node.name])

    input2_node = node_def_pb2.NodeDef()
    input2_node.name = "input2"
    input2_node.op = "Const"
    input2_value = np.float32(np.abs(np.random.randn(1)))
    input2_node.attr["value"].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            input2_value, input2_value.dtype.type, input2_value.shape)))

    input3_node = node_def_pb2.NodeDef()
    input3_node.name = "input3"
    input3_node.op = "Const"
    input3_value = np.float32(np.abs(np.random.randn(1)))
    input3_node.attr["value"].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            input3_value, input3_value.dtype.type, input3_value.shape)))

    switch_node = node_def_pb2.NodeDef()
    switch_node.name = "switch"
    switch_node.op = "Switch"

    input4_node = node_def_pb2.NodeDef()
    input4_node.name = "input4"
    input4_node.op = "Const"
    input4_value = np.float32(np.abs(np.random.randn(1)))
    input4_node.attr["value"].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            input4_value, input4_value.dtype.type, input4_value.shape)))
    input4_node.input.extend([switch_node.name])

    input5_node = node_def_pb2.NodeDef()
    input5_node.name = "input5"
    input5_node.op = "Const"
    input5_value = np.float32(np.abs(np.random.randn(1)))
    input5_node.attr["value"].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            input5_value, input5_value.dtype.type, input5_value.shape)))
    input5_node.input.extend([switch_node.name])

    cond_end = node_def_pb2.NodeDef()
    cond_end.name = "cond"
    cond_end.op = "Add"
    cond_end.input.extend([input4_node.name, input5_node.name])

    mul_node = node_def_pb2.NodeDef()
    mul_node.op = "Mul"
    mul_node.name = "mul"
    mul_node.input.extend([add_node.name, input3_node.name])

    sqrt_node = node_def_pb2.NodeDef()
    sqrt_node.name = "rsqrt"
    sqrt_node.op = "Rsqrt"
    sqrt_node.input.extend([mul_node.name])

    relu_node = node_def_pb2.NodeDef()
    relu_node.op = "Relu"
    relu_node.name = "relu"
    relu_node.input.extend([sqrt_node.name])

    block_node = node_def_pb2.NodeDef()
    block_node.name = "block_output"
    block_node.op = "Add"
    block_node.input.extend([x_node.name, relu_node.name])

    res_node = node_def_pb2.NodeDef()
    res_node.name = "res_add"
    res_node.op = "Add"
    res_node.input.extend([sqrt_node.name, input2_node.name])

    end_node = node_def_pb2.NodeDef()
    end_node.name = "end"
    end_node.op = "Add"
    end_node.input.extend([block_node.name, res_node.name])

    graph_def = graph_pb2.GraphDef()
    graph_def.node.extend([
        x_node, input0_node, input1_node, input2_node, input3_node, add_node, mul_node, sqrt_node,
        relu_node, block_node, res_node, end_node
    ])

    def test_fold_constant(self):

        graph = self.graph_def
        rewriter = GraphFoldConstantOptimizer(graph)
        new_graph = rewriter.do_transformation()

        for node in new_graph.node:
            assert node.name in [
                "placeholder", "block_output", "rsqrt_const", "relu", "res_add_const", "end"
            ]

    def test_condition_fold_constant(self):
        graph_def = graph_pb2.GraphDef()
        graph_def.node.extend([self.cond_end, self.input4_node,
                               self.input5_node, self.switch_node])
        rewriter = GraphFoldConstantOptimizer(graph_def)
        new_graph = rewriter.do_transformation()
        for node in new_graph.node:
            assert node.name in ["switch", "cond", "input4", "input5"]

    def test_slice_int_input(self):
        graph_def = graph_pb2.GraphDef()
        index0_node = node_def_pb2.NodeDef()
        index0_node.name = "index0"
        index0_node.op = "Const"
        index0_value = np.array(3).astype(np.int32).reshape(())
        index0_node.attr["value"].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            index0_value, index0_value.dtype.type, index0_value.shape)))

        index1_node = node_def_pb2.NodeDef()
        index1_node.name = "index1"
        index1_node.op = "Const"
        index1_value = np.array(1).astype(np.int32).reshape(())
        index1_node.attr["value"].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            index1_value, index1_value.dtype.type, index1_value.shape)))

        minus_node = node_def_pb2.NodeDef()
        minus_node.name = "sub"
        minus_node.op = "Sub"
        minus_node.input.extend([index0_node.name, index1_node.name])

        graph_def.node.extend([index0_node, index1_node, minus_node])
        rewriter = GraphFoldConstantOptimizer(graph_def)
        new_graph = rewriter.do_transformation()
        with tf.compat.v1.Session() as sess:
            tf.compat.v1.import_graph_def(new_graph)

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Consts test."""

import unittest

from lpot.ux.utils.consts import github_info


class TestConsts(unittest.TestCase):
    """Consts tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Consts tests constructor."""
        super().__init__(*args, **kwargs)

    def test_github_info(self) -> None:
        """Test if path is correctly recognized as hidden."""
        self.assertIs(type(github_info), dict)
        self.assertEqual(github_info.get("user"), "intel")
        self.assertEqual(github_info.get("repository"), "lpot")
        self.assertEqual(github_info.get("tag"), "v1.5")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import abstractmethod
from lpot.utils.utility import LazyImport, singleton

torch = LazyImport('torch')
tf = LazyImport('tensorflow')

@singleton
class TensorflowCriterions(object):
    def __init__(self):
        self.criterions = {}
        self.criterions.update(TENSORFLOW_CRITERIONS)

@singleton
class PyTorchCriterions(object):
    def __init__(self):
        self.criterions = {}
        self.criterions.update(PYTORCH_CRITERIONS)

framework_criterions = {"tensorflow": TensorflowCriterions,
                        "pytorch":    PyTorchCriterions}

# user/model specific criterions will be registered here
TENSORFLOW_CRITERIONS = {}
PYTORCH_CRITERIONS= {}

registry_criterions = {"tensorflow": TENSORFLOW_CRITERIONS,
                       "pytorch":    PYTORCH_CRITERIONS}

class Criterions(object):
    def __init__(self, framework):
        assert framework in ("tensorflow", "pytorch"), \
                             "framework support tensorflow pytorch"
        self.criterions = framework_criterions[framework]().criterions

    def __getitem__(self, criterion_type):
        assert criterion_type in self.criterions.keys(), "only support criterions in {}".\
            format(self.criterions.keys())

        return self.criterions[criterion_type]

    def register(self, name, criterion_cls):
        assert name not in self.criterions.keys(), 'registered criterion name already exists.'
        self.criterions.update({name: criterion_cls})

def criterion_registry(criterion_type, framework):
    """The class decorator used to register all criterion classes.

    Args:
        criterion_type (str): The string of supported criterion.
        framework (str): The string of supported framework. 

    Returns:
        cls: The class of register.
    """
    def decorator_criterion(cls):
        for fw in [fwk.strip() for fwk in framework.split(',')]:
            assert fw in [
                "tensorflow",
                "pytorch"], "The framework support tensorflow pytorch"

            if criterion_type in registry_criterions[fw].keys():
                raise ValueError('Cannot have two criterions with the same name')
            registry_criterions[fw][criterion_type] = cls
        return cls
    return decorator_criterion

@criterion_registry('CrossEntropyLoss', 'tensorflow')
class TensorFlowCrossEntropyLoss(object):
    """TensorFlow CrossEntropyLoss criterion.

    Args:
        param_dict (dict): The dict of parameters setting by user for CrossEntropyLoss criterion.
    """
    def __init__(self, param_dict):
        assert isinstance(param_dict, dict), 'This criterion constructor parameter must be a dict'
        self._param_dict = param_dict

    def _mapping(self):
        _param_map = {'reduction': 'reduction'}
        _dict = {}
        for key in self._param_dict:
            if key in _param_map:
                if key == 'reduction':
                    assert self._param_dict[key] in ['none', 'mean', 'sum'], \
                        'Supported reduction value is none, mean, sum'
                _dict.update({_param_map[key]: self._param_dict[key]})
        return _dict

    def __call__(self, **kwargs):
        return tf.keras.losses.CategoricalCrossentropy, self._mapping(**kwargs)

@criterion_registry('CrossEntropyLoss', 'pytorch')
class PyTorchCrossEntropyLoss(object):
    """PyTorch CrossEntropyLoss criterion.

    Args:
        param_dict (dict): The dict of parameters setting by user for SGD criterion
    """
    def __init__(self, param_dict):
        assert isinstance(param_dict, dict), 'This criterion constructor parameter must be a dict'
        self._param_dict = param_dict

    def _mapping(self):
        _param_map = {'reduction': 'reduction'}
        _dict = {}
        for key in self._param_dict:
            if key in _param_map:
                if key == 'reduction':
                    assert self._param_dict[key] in ['none', 'mean', 'sum'], \
                        'Supported reduction value is none, mean, sum'
                _dict.update({_param_map[key]: self._param_dict[key]})
        return _dict

    def __call__(self, **kwargs):
        return torch.nn.CrossEntropyLoss, self._mapping(**kwargs)



import unittest
import tensorflow as tf
import numpy as np
import os
import json
import shutil
from PIL import Image
from lpot.data import FILTERS, TRANSFORMS, DATASETS, DATALOADERS
from lpot.utils.create_obj_from_config import create_dataset, get_preprocess, create_dataloader

class TestCOCOFilter(unittest.TestCase):
    def testLabelBalanceCOCORecord(self):
        tf.compat.v1.disable_eager_execution() 

        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        im.save('test.jpeg')

        image = tf.compat.v1.gfile.FastGFile('test.jpeg','rb').read()
        source_id = '000000397133.jpg'.encode('utf-8')
        label = 'person'.encode('utf-8')
        example1 = tf.train.Example(features=tf.train.Features(feature={
            'image/encoded':tf.train.Feature(
                    bytes_list=tf.train.BytesList(value=[image])),
            'image/object/class/text':tf.train.Feature(
                    bytes_list=tf.train.BytesList(value=[label])),
            'image/source_id':tf.train.Feature(
                    bytes_list=tf.train.BytesList(value=[source_id])),
            'image/object/bbox/xmin':tf.train.Feature(
                    float_list=tf.train.FloatList(value=[10])),
            'image/object/bbox/ymin':tf.train.Feature(
                    float_list=tf.train.FloatList(value=[10])),
            'image/object/bbox/xmax':tf.train.Feature(
                    float_list=tf.train.FloatList(value=[100])),
            'image/object/bbox/ymax':tf.train.Feature(
                    float_list=tf.train.FloatList(value=[100])),
        }))
        example2 = tf.train.Example(features=tf.train.Features(feature={
            'image/encoded':tf.train.Feature(
                    bytes_list=tf.train.BytesList(value=[image])),
            'image/object/class/text':tf.train.Feature(
                    bytes_list=tf.train.BytesList(value=[label])),
            'image/source_id':tf.train.Feature(
                    bytes_list=tf.train.BytesList(value=[source_id])),
            'image/object/bbox/xmin':tf.train.Feature(
                    float_list=tf.train.FloatList(value=[10, 20])),
            'image/object/bbox/ymin':tf.train.Feature(
                    float_list=tf.train.FloatList(value=[10, 20])),
            'image/object/bbox/xmax':tf.train.Feature(
                    float_list=tf.train.FloatList(value=[100, 200])),
            'image/object/bbox/ymax':tf.train.Feature(
                    float_list=tf.train.FloatList(value=[100, 200])),
        }))
        with tf.io.TFRecordWriter('test.record') as writer:
            writer.write(example1.SerializeToString())
            writer.write(example2.SerializeToString())
        
        preprocesses = TRANSFORMS('tensorflow', 'preprocess')
        filters = FILTERS('tensorflow')
        filter = filters['LabelBalanceCOCORecord'](2)
        datasets = DATASETS('tensorflow')
        dataset = datasets['COCORecord']('test.record', \
            transform=None, filter=filter)
        dataloader = DATALOADERS['tensorflow'](dataset=dataset, batch_size=1)
        for (inputs, labels) in dataloader:
            self.assertEqual(inputs.shape, (1,100,100,3))
            self.assertEqual(labels[0].shape, (1,2,4))

        dataset2 = create_dataset(
            'tensorflow', {'COCORecord':{'root':'test.record'}}, None, {'LabelBalance':{'size':2}})
        dataloader2 = DATALOADERS['tensorflow'](dataset=dataset2, batch_size=1)
        for (inputs, labels) in dataloader2:
            self.assertEqual(inputs.shape, (1,100,100,3))
            self.assertEqual(labels[0].shape, (1,2,4))

        dataloader3 = create_dataloader('tensorflow', {'batch_size':1, 'dataset':{'COCORecord':{'root':'test.record'}},\
                 'filter':{'LabelBalance':{'size':2}}, 'transform':None})
        for (inputs, labels) in dataloader3:
            self.assertEqual(inputs.shape, (1,100,100,3))
            self.assertEqual(labels[0].shape, (1,2,4))
        os.remove('test.record')
        os.remove('test.jpeg')

    def testLabelBalanceCOCORaw(self):
        random_array = np.random.random_sample([100,100,3]) * 255
        random_array = random_array.astype(np.uint8)
        im = Image.fromarray(random_array)
        os.makedirs('val2017', exist_ok=True)
        im.save('./val2017/test_0.jpg')
        im.save('./val2017/test_1.jpg')
        fake_dict = {
            'info': {
                'description': 'COCO 2017 Dataset',
                'url': 'http://cocodataset.org',
                'version': '1.0',
                'year': 2017,
                'contributor': 'COCO Consortium',
                'date_created': '2017/09/01'
            },
            'licenses':{},
            'images':[{
                'file_name': 'test_0.jpg',
                'height': 100,
                'width': 100,
                'id': 0
            },
            {
                'file_name': 'test_1.jpg',
                'height': 100,
                'width': 100,
                'id': 1
            }],
            'annotations':[{
                'category_id': 18,
                'id': 1767,
                'iscrowd': 0,
                'image_id': 0,
                'bbox': [473.07, 395.93, 38.65, 28.67],
            },
            {
               'category_id': 18,
               'id': 1768,
               'iscrowd': 0,
               'image_id': 1,
               'bbox': [473.07, 395.93, 38.65, 28.67],
            },
            {
               'category_id': 18,
               'id': 1768,
               'iscrowd': 0,
               'image_id': 1,
               'bbox': [473.07, 395.93, 38.65, 28.67],
            }],
            'categories':[{
                'supercategory': 'animal',
                'id': 18,
                'name': 'dog'
            }]
        }
        fake_json = json.dumps(fake_dict)
        os.makedirs('annotations', exist_ok=True)
        with open('./annotations/instances_val2017.json', 'w') as f:
            f.write(fake_json)

        filters = FILTERS('onnxrt_qlinearops')
        filter = filters['LabelBalanceCOCORaw'](1)
        datasets = DATASETS('onnxrt_qlinearops')
        dataset = datasets['COCORaw']('./', transform=None, filter=filter)
        dataloader = DATALOADERS['onnxrt_qlinearops'](dataset=dataset, batch_size=1)
        for (inputs, labels) in dataloader:
            self.assertEqual(labels[0].shape[1], 1)

        shutil.rmtree('annotations')
        shutil.rmtree('val2017')

if __name__ == "__main__":
    unittest.main()




# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Quantization config test."""

import unittest

from lpot.ux.utils.workload.quantization import (
    Advance,
    Calibration,
    Quantization,
    WiseConfig,
    WiseConfigDetails,
)


class TestCalibrationConfig(unittest.TestCase):
    """Calibration config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Calibration config test constructor."""
        super().__init__(*args, **kwargs)

    def test_calibration_constructor(self) -> None:
        """Test Calibration config constructor."""
        data = {
            "sampling_size": "10, 50, 100, 200",
            "dataloader": {
                "last_batch": "rollover",
                "batch_size": 2,
                "dataset": {
                    "TestDataset": {
                        "dataset_param": "/some/path",
                        "bool_param": True,
                        "list_param": ["item1", "item2"],
                    },
                },
                "transform": {
                    "TestTransform": {"shape": [1000, 224, 224, 3], "some_op": True},
                    "AnotherTestTransform": {
                        "shape": [10, 299, 299, 3],
                        "some_op": False,
                    },
                },
                "filter": {
                    "LabelBalance": {"size": 1},
                },
            },
        }
        calibration = Calibration(data)

        self.assertEqual(calibration.sampling_size, "10, 50, 100, 200")
        self.assertEqual(calibration.dataloader.last_batch, "rollover")
        self.assertEqual(
            calibration.dataloader.batch_size,
            1,
        )  # Calibration batch size should be always set to 1
        self.assertIsNotNone(calibration.dataloader.dataset)
        self.assertEqual(calibration.dataloader.dataset.name, "TestDataset")
        self.assertDictEqual(
            calibration.dataloader.dataset.params,
            {
                "dataset_param": "/some/path",
                "bool_param": True,
                "list_param": ["item1", "item2"],
            },
        )
        transform_name, transform = list(calibration.dataloader.transform.items())[0]
        self.assertEqual(transform_name, "TestTransform")
        self.assertEqual(transform.name, "TestTransform")
        self.assertDictEqual(
            transform.parameters,
            {
                "shape": [1000, 224, 224, 3],
                "some_op": True,
            },
        )
        transform_name, transform = list(calibration.dataloader.transform.items())[1]
        self.assertEqual(transform_name, "AnotherTestTransform")
        self.assertEqual(transform.name, "AnotherTestTransform")
        self.assertDictEqual(
            transform.parameters,
            {"shape": [10, 299, 299, 3], "some_op": False},
        )
        self.assertIsNotNone(calibration.dataloader.filter)
        self.assertIsNotNone(calibration.dataloader.filter.LabelBalance)
        self.assertEqual(calibration.dataloader.filter.LabelBalance.size, 1)

    def test_calibration_constructor_defaults(self) -> None:
        """Test Calibration config constructor defaults."""
        calibration = Calibration()

        self.assertEqual(calibration.sampling_size, 100)
        self.assertIsNone(calibration.dataloader)

    def test_calibration_serializer(self) -> None:
        """Test Calibration config serializer."""
        data = {
            "sampling_size": "10, 50, 100, 200",
            "dataloader": {
                "last_batch": "rollover",
                "batch_size": 2,
                "dataset": {
                    "TestDataset": {
                        "dataset_param": "/some/path",
                        "bool_param": True,
                        "list_param": ["item1", "item2"],
                    },
                },
                "transform": {
                    "TestTransform": {"shape": [1000, 224, 224, 3], "some_op": True},
                    "AnotherTestTransform": {
                        "shape": [10, 299, 299, 3],
                        "some_op": False,
                    },
                },
                "filter": {
                    "LabelBalance": {"size": 1},
                },
                "some_additional_field": {
                    "not_important_data": True,
                },
            },
        }
        calibration = Calibration(data)
        result = calibration.serialize()

        self.assertDictEqual(
            result,
            {
                "sampling_size": "10, 50, 100, 200",
                "dataloader": {
                    "last_batch": "rollover",
                    "batch_size": 1,
                    "dataset": {
                        "TestDataset": {
                            "dataset_param": "/some/path",
                            "bool_param": True,
                            "list_param": ["item1", "item2"],
                        },
                    },
                    "transform": {
                        "TestTransform": {
                            "shape": [1000, 224, 224, 3],
                            "some_op": True,
                        },
                        "AnotherTestTransform": {
                            "shape": [10, 299, 299, 3],
                            "some_op": False,
                        },
                    },
                    "filter": {
                        "LabelBalance": {"size": 1},
                    },
                },
            },
        )


class TestWiseConfigDetailsConfig(unittest.TestCase):
    """WiseConfigDetails config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Initialize WiseConfigDetails config test."""
        super().__init__(*args, **kwargs)

    def test_wise_config_details_constructor(self) -> None:
        """Test WiseConfigDetails config constructor."""
        data = {
            "granularity": "per_channel",
            "scheme": "asym",
            "dtype": "bf16",
            "algorithm": "kl",
        }
        wise_config_details = WiseConfigDetails(data)

        self.assertEqual(wise_config_details.granularity, "per_channel")
        self.assertEqual(wise_config_details.scheme, "asym")
        self.assertEqual(wise_config_details.dtype, "bf16")
        self.assertEqual(wise_config_details.algorithm, "kl")

    def test_wise_config_details_constructor_defaults(self) -> None:
        """Test WiseConfigDetails config constructor defaults."""
        wise_config_details = WiseConfigDetails()

        self.assertIsNone(wise_config_details.granularity)
        self.assertIsNone(wise_config_details.scheme)
        self.assertIsNone(wise_config_details.dtype)
        self.assertIsNone(wise_config_details.algorithm)

    def test_wise_config_details_serializer(self) -> None:
        """Test WiseConfigDetails config serializer."""
        data = {
            "granularity": "per_channel",
            "scheme": "asym",
            "dtype": "bf16",
            "algorithm": "kl",
            "output_dtype": "int8",
            "wise": True,
        }
        wise_config_details = WiseConfigDetails(data)
        result = wise_config_details.serialize()

        self.assertDictEqual(
            result,
            {
                "granularity": "per_channel",
                "scheme": "asym",
                "dtype": "bf16",
                "algorithm": "kl",
            },
        )


class TestWiseConfigConfig(unittest.TestCase):
    """WiseConfig config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Initialize WiseConfig config test."""
        super().__init__(*args, **kwargs)

    def test_wise_config_constructor(self) -> None:
        """Test WiseConfig config constructor."""
        data = {
            "weight": {
                "granularity": "per_channel",
                "scheme": "asym",
                "dtype": "bf16",
                "algorithm": "minmax",
            },
            "activation": {
                "granularity": "per_tensor",
                "scheme": "sym",
                "dtype": "int8",
                "algorithm": "minmax",
            },
        }
        wise_config = WiseConfig(data)

        self.assertIsNotNone(wise_config.weight)
        self.assertEqual(wise_config.weight.granularity, "per_channel")
        self.assertEqual(wise_config.weight.scheme, "asym")
        self.assertEqual(wise_config.weight.dtype, "bf16")
        self.assertEqual(wise_config.weight.algorithm, "minmax")

        self.assertIsNotNone(wise_config.activation)
        self.assertEqual(wise_config.activation.granularity, "per_tensor")
        self.assertEqual(wise_config.activation.scheme, "sym")
        self.assertEqual(wise_config.activation.dtype, "int8")
        self.assertEqual(wise_config.activation.algorithm, "minmax")

    def test_wise_config_constructor_defaults(self) -> None:
        """Test WiseConfig config constructor defaults."""
        wise_config = WiseConfig()

        self.assertIsNone(wise_config.weight)
        self.assertIsNone(wise_config.activation)

    def test_wise_config_serializer(self) -> None:
        """Test WiseConfig config serializer."""
        data = {
            "weight": {
                "granularity": "per_channel",
                "scheme": "asym",
                "dtype": "bf16",
                "algorithm": "minmax",
            },
            "activation": {
                "granularity": "per_tensor",
                "scheme": "sym",
                "dtype": "int8",
                "algorithm": "minmax",
            },
            "additional_field": {"asd": 123},
        }
        wise_config = WiseConfig(data)
        result = wise_config.serialize()

        self.assertDictEqual(
            result,
            {
                "weight": {
                    "granularity": "per_channel",
                    "scheme": "asym",
                    "dtype": "bf16",
                    "algorithm": "minmax",
                },
                "activation": {
                    "granularity": "per_tensor",
                    "scheme": "sym",
                    "dtype": "int8",
                    "algorithm": "minmax",
                },
            },
        )


class TestAdvanceConfig(unittest.TestCase):
    """Advance config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Advance config test constructor."""
        super().__init__(*args, **kwargs)

    def test_advance_constructor(self) -> None:
        """Test Advance config constructor."""
        data = {"bias_correction": "weight_empirical"}
        advance = Advance(data)

        self.assertEqual(advance.bias_correction, "weight_empirical")

    def test_advance_constructor_defaults(self) -> None:
        """Test Advance config constructor defaults."""
        advance = Advance()

        self.assertIsNone(advance.bias_correction)


class TestQuantizationConfig(unittest.TestCase):
    """Quantization config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Quantization config test constructor."""
        super().__init__(*args, **kwargs)

    def test_quantization_constructor(self) -> None:
        """Test Quantization config constructor."""
        data = {
            "calibration": {
                "sampling_size": "10, 50, 100, 200",
                "dataloader": {
                    "last_batch": "rollover",
                    "batch_size": 2,
                    "dataset": {
                        "TestDataset": {
                            "dataset_param": "/some/path",
                            "bool_param": True,
                            "list_param": ["item1", "item2"],
                        },
                    },
                    "transform": {
                        "TestTransform": {
                            "shape": [1000, 224, 224, 3],
                            "some_op": True,
                        },
                        "AnotherTestTransform": {
                            "shape": [10, 299, 299, 3],
                            "some_op": False,
                        },
                    },
                    "filter": {
                        "LabelBalance": {"size": 1},
                    },
                },
            },
            "model_wise": {
                "weight": {
                    "granularity": "per_channel_model",
                    "scheme": "asym_model",
                    "dtype": "bf16_model",
                    "algorithm": "kl_model",
                },
                "activation": {
                    "granularity": "per_tensor_model",
                    "scheme": "sym_model",
                    "dtype": "int8_model",
                    "algorithm": "minmax_model",
                },
            },
            "op_wise": {
                "weight": {
                    "granularity": "per_channel_op",
                    "scheme": "asym_op",
                    "dtype": "bf16_op",
                    "algorithm": "kl_op",
                },
                "activation": {
                    "granularity": "per_tensor_op",
                    "scheme": "sym_op",
                    "dtype": "int8_op",
                    "algorithm": "minmax_op",
                },
            },
            "approach": "quant_aware_training",
            "advance": {"bias_correction": "weight_empirical"},
        }
        quantization = Quantization(data)

        self.assertIsNotNone(quantization.calibration)
        self.assertEqual(quantization.calibration.sampling_size, "10, 50, 100, 200")
        self.assertEqual(quantization.calibration.dataloader.last_batch, "rollover")
        self.assertEqual(
            quantization.calibration.dataloader.batch_size,
            1,
        )  # Calibration batch size should be always set to 1
        self.assertIsNotNone(quantization.calibration.dataloader.dataset)
        self.assertEqual(
            quantization.calibration.dataloader.dataset.name,
            "TestDataset",
        )
        self.assertDictEqual(
            quantization.calibration.dataloader.dataset.params,
            {
                "dataset_param": "/some/path",
                "bool_param": True,
                "list_param": ["item1", "item2"],
            },
        )
        transform_name, transform = list(
            quantization.calibration.dataloader.transform.items(),
        )[0]
        self.assertEqual(transform_name, "TestTransform")
        self.assertEqual(transform.name, "TestTransform")
        self.assertDictEqual(
            transform.parameters,
            {
                "shape": [1000, 224, 224, 3],
                "some_op": True,
            },
        )
        transform_name, transform = list(
            quantization.calibration.dataloader.transform.items(),
        )[1]
        self.assertEqual(transform_name, "AnotherTestTransform")
        self.assertEqual(transform.name, "AnotherTestTransform")
        self.assertDictEqual(
            transform.parameters,
            {"shape": [10, 299, 299, 3], "some_op": False},
        )
        self.assertIsNotNone(quantization.calibration.dataloader.filter)
        self.assertIsNotNone(
            quantization.calibration.dataloader.filter.LabelBalance,
        )
        self.assertEqual(
            quantization.calibration.dataloader.filter.LabelBalance.size,
            1,
        )

        self.assertIsNotNone(quantization.model_wise)
        self.assertIsNotNone(quantization.model_wise.weight)
        self.assertEqual(
            quantization.model_wise.weight.granularity,
            "per_channel_model",
        )
        self.assertEqual(quantization.model_wise.weight.scheme, "asym_model")
        self.assertEqual(quantization.model_wise.weight.dtype, "bf16_model")
        self.assertEqual(quantization.model_wise.weight.algorithm, "kl_model")
        self.assertIsNotNone(quantization.model_wise.activation)
        self.assertEqual(
            quantization.model_wise.activation.granularity,
            "per_tensor_model",
        )
        self.assertEqual(quantization.model_wise.activation.scheme, "sym_model")
        self.assertEqual(quantization.model_wise.activation.dtype, "int8_model")
        self.assertEqual(quantization.model_wise.activation.algorithm, "minmax_model")

        self.assertIsNotNone(quantization.op_wise)
        self.assertDictEqual(
            quantization.op_wise,
            {
                "weight": {
                    "granularity": "per_channel_op",
                    "scheme": "asym_op",
                    "dtype": "bf16_op",
                    "algorithm": "kl_op",
                },
                "activation": {
                    "granularity": "per_tensor_op",
                    "scheme": "sym_op",
                    "dtype": "int8_op",
                    "algorithm": "minmax_op",
                },
            },
        )

        self.assertEqual(quantization.approach, "quant_aware_training")

        self.assertIsNotNone(quantization.advance)
        self.assertEqual(quantization.advance.bias_correction, "weight_empirical")

    def test_quantization_constructor_defaults(self) -> None:
        """Test Quantization config constructor defaults."""
        quantization = Quantization()

        self.assertIsNone(quantization.calibration)
        self.assertIsNone(quantization.model_wise)
        self.assertIsNone(quantization.op_wise)
        self.assertEqual(quantization.approach, "post_training_static_quant")
        self.assertIsNone(quantization.advance)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Connector between api.py and components."""

from threading import Thread
from typing import Any, Callable, Dict

from werkzeug.wrappers import BaseResponse as WebResponse

from lpot.ux.components.benchmark.execute_benchmark import execute_benchmark
from lpot.ux.components.configuration_wizard.get_boundary_nodes import get_boundary_nodes
from lpot.ux.components.configuration_wizard.get_configuration import get_predefined_configuration
from lpot.ux.components.configuration_wizard.params_feeder import get_possible_values
from lpot.ux.components.configuration_wizard.save_workload import save_workload
from lpot.ux.components.file_browser.file_browser import get_directory_entries
from lpot.ux.components.graph.graph import Graph
from lpot.ux.components.graph.graph_reader import GraphReader
from lpot.ux.components.manage_workspace import (
    delete_workload,
    get_default_path,
    get_workloads_list,
    set_workspace,
)
from lpot.ux.components.model_zoo.download_config import download_config
from lpot.ux.components.model_zoo.download_model import download_model
from lpot.ux.components.model_zoo.list_models import list_models
from lpot.ux.components.optimization.execute_optimization import execute_optimization
from lpot.ux.utils.hw_info import HWInfo
from lpot.ux.utils.json_serializer import JsonSerializer
from lpot.ux.web.communication import Request, Response, create_simple_response
from lpot.ux.web.exceptions import ServiceNotFoundException
from lpot.ux.web.service.request_data_processor import RequestDataProcessor
from lpot.ux.web.service.workload import WorkloadService


class RoutingDefinition:
    """Abstract routing definition."""

    def __init__(self, callback: Callable[[Dict[str, Any]], Any]) -> None:
        """Initialize object."""
        self.callback = callback


class RealtimeRoutingDefinition(RoutingDefinition):
    """Routing executed in realtime."""

    pass


class DeferredRoutingDefinition(RoutingDefinition):
    """Routing executed in separate thread."""

    pass


class Router:
    """Connector between api.py and components."""

    def __init__(self) -> None:
        """Initialize object."""
        self.routes: Dict[str, RoutingDefinition] = {
            "filesystem": RealtimeRoutingDefinition(get_directory_entries),
            "save_workload": RealtimeRoutingDefinition(save_workload),
            "delete_workload": RealtimeRoutingDefinition(delete_workload),
            "configuration": RealtimeRoutingDefinition(get_predefined_configuration),
            "optimize": DeferredRoutingDefinition(_execute_optimization_benchmark),
            "benchmark": DeferredRoutingDefinition(execute_benchmark),
            "get_default_path": RealtimeRoutingDefinition(get_default_path),
            "set_workspace": RealtimeRoutingDefinition(set_workspace),
            "get_workloads_list": RealtimeRoutingDefinition(get_workloads_list),
            "get_boundary_nodes": DeferredRoutingDefinition(get_boundary_nodes),
            "get_possible_values": RealtimeRoutingDefinition(get_possible_values),
            "download_model": DeferredRoutingDefinition(download_model),
            "list_model_zoo": RealtimeRoutingDefinition(list_models),
            "download_config": DeferredRoutingDefinition(download_config),
            "model_graph": RealtimeRoutingDefinition(get_model_graph),
            "system_info": RealtimeRoutingDefinition(get_system_info),
            "workload/config.yaml": RealtimeRoutingDefinition(WorkloadService.get_config),
            "workload/output.log": RealtimeRoutingDefinition(WorkloadService.get_output),
            "workload/code_template.py": RealtimeRoutingDefinition(
                WorkloadService.get_code_template,
            ),
        }

    def handle(self, request: Request) -> Response:
        """Run operation on requested component and return result."""
        routing_definition = self.routes.get(request.operation)
        if routing_definition is None:
            raise ServiceNotFoundException(f"Unable to find {request.operation}")

        data = self._process_routing_definition(routing_definition, request.data)
        if isinstance(data, WebResponse):
            return data

        serialized_data = JsonSerializer.serialize_item(data)

        return create_simple_response(serialized_data)

    @staticmethod
    def _process_routing_definition(
        routing_definition: RoutingDefinition,
        data: dict,
    ) -> Any:
        """Process routing definition."""
        if isinstance(routing_definition, RealtimeRoutingDefinition):
            return routing_definition.callback(data)
        if isinstance(routing_definition, DeferredRoutingDefinition):
            t = Thread(target=routing_definition.callback, args=(data,))
            t.daemon = True
            t.start()
            return {"exit_code": 102, "message": "processing"}
        raise ValueError(
            f"Unsupported RoutingDefinition type: {routing_definition.__class__.__name__}",
        )


def _execute_optimization_benchmark(data: dict) -> None:
    """Execute both tuning and benchmark."""
    optimization_data = execute_optimization(data)
    models_info = optimization_data.get("execution_details", {}).get("optimization", {})
    benchmark_data = {
        "id": data.get("id"),
        "input_model": {
            "precision": models_info.get("input_precision"),
            "path": models_info.get("input_graph"),
        },
        "optimized_model": {
            "precision": models_info.get("output_precision"),
            "path": models_info.get("output_graph"),
        },
        "workspace_path": data.get("workspace_path"),
    }
    if not optimization_data.get("is_custom_dataloader", None):
        execute_benchmark(benchmark_data)


def get_model_graph(data: Dict[str, Any]) -> Graph:
    """Get model graph."""
    graph_reader = GraphReader()
    return graph_reader.read(
        model_path=RequestDataProcessor.get_string_value(data, "path"),
        expanded_groups=data.get("group", []),
    )


def get_system_info(data: Dict[str, Any]) -> dict:
    """Get system info."""
    hw_info = vars(HWInfo())
    if "cores" in hw_info:
        del hw_info["cores"]
    return hw_info




import unittest
import os
import yaml
import tensorflow as tf
from tensorflow.python.framework import graph_util
from lpot.adaptor.tf_utils.util import disable_random


def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: input
          outputs: op_to_store
        device: cpu
        quantization:
          model_wise:
            weight:
                granularity: per_tensor
                scheme: sym
                dtype: int8
                algorithm: minmax
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            strategy:
              name: mse
            accuracy_criterion:
              relative: 0.01
            exit_policy:
              performance_only: True
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()


class TestFoldPadConv(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        build_fake_yaml()

    @classmethod
    def tearDownClass(self):
        os.remove('fake_yaml.yaml')

    @disable_random()
    def test_fold_pad_conv(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(x, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        normed = tf.compat.v1.layers.batch_normalization(conv)
        relu = tf.nn.relu(normed, name='op_to_store')
        out_name = relu.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])

            from lpot.experimental import Quantization, common
            quantizer = Quantization('fake_yaml.yaml')
            dataset = quantizer.dataset('dummy', shape=(100, 56, 56, 16), label=True)
            quantizer.eval_dataloader = common.DataLoader(dataset)
            quantizer.calib_dataloader = common.DataLoader(dataset)
            quantizer.model = output_graph_def
            output_graph = quantizer()
            found_pad = False

            if tf.__version__ >= "2.0.0":
                for i in output_graph.graph_def.node:
                    if i.op == 'Pad':
                        found_pad = True
                        break
                self.assertEqual(found_pad, True)

    @disable_random()
    def test_fold_pad_conv2(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(x, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        normed = tf.compat.v1.layers.batch_normalization(conv)
        relu = tf.nn.relu(normed)

        paddings2 = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad2 = tf.pad(x, paddings2, "CONSTANT")
        conv_weights2 = tf.compat.v1.get_variable("weight2", [3, 3, 16, 16],
                                                  initializer=tf.compat.v1.random_normal_initializer())
        conv2 = tf.nn.conv2d(x_pad2, conv_weights2, strides=[1, 2, 2, 1], padding="VALID")
        normed2 = tf.compat.v1.layers.batch_normalization(conv2)
        relu2 = tf.nn.relu(normed2)
        add = tf.math.add(relu, relu2, name='op_to_store')
        out_name = add.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from lpot.experimental import Quantization, common
            quantizer = Quantization('fake_yaml.yaml')
            dataset = quantizer.dataset('dummy', shape=(100, 56, 56, 16), label=True)
            quantizer.eval_dataloader = common.DataLoader(dataset)
            quantizer.calib_dataloader = common.DataLoader(dataset)
            quantizer.model = output_graph_def
            output_graph = quantizer()
            found_pad = False

            if tf.__version__ >= "2.0.0":
                for i in output_graph.graph_def.node:
                    if i.op == 'Pad':
                        found_pad = True
                        break
                self.assertEqual(found_pad, True)

    @disable_random()
    def test_fold_pad_conv3(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(x, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        normed = tf.compat.v1.layers.batch_normalization(conv)
        relu = tf.nn.relu(normed)

        conv_weights2 = tf.compat.v1.get_variable("weight2", [3, 3, 16, 16],
                                                  initializer=tf.compat.v1.random_normal_initializer())
        conv2 = tf.nn.conv2d(x, conv_weights2, strides=[1, 2, 2, 1], padding="SAME")
        normed2 = tf.compat.v1.layers.batch_normalization(conv2)
        relu2 = tf.nn.relu(normed2)
        add = tf.math.add(relu, relu2, name='op_to_store')
        out_name = add.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from lpot.experimental import Quantization, common
            quantizer = Quantization('fake_yaml.yaml')
            dataset = quantizer.dataset('dummy', shape=(100, 56, 56, 16), label=True)
            quantizer.eval_dataloader = common.DataLoader(dataset)
            quantizer.calib_dataloader = common.DataLoader(dataset)
            quantizer.model = output_graph_def
            output_graph = quantizer()
            found_pad = False

            if tf.__version__ >= "2.0.0":
                for i in output_graph.graph_def.node:
                    if i.op == 'Pad':
                        found_pad = True
                        break

                self.assertEqual(found_pad, True)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Executor test."""

import os
import shutil
import unittest

from lpot.ux.utils.executor import Executor


class TestExecutor(unittest.TestCase):
    """Executor tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Executor tests constructor."""
        super().__init__(*args, **kwargs)
        self.executor = Executor(
            workspace_path="tmp_workdir",
            subject="test",
            data={"id": "abc", "some_key": "some_value"},
            send_response=False,
            log_name="my_log",
            additional_log_names=["additional_log1", "additional_log2"],
        )

    @classmethod
    def tearDownClass(cls) -> None:
        """Tear down environment for test."""
        shutil.rmtree("tmp_workdir", ignore_errors=True)

    def test_workdir_property(self) -> None:
        """Test if workdir property returns correct path."""
        self.assertEqual(self.executor.workdir, "tmp_workdir")

    def test_request_id_property(self) -> None:
        """Test if request_id property returns correct value."""
        self.assertEqual(self.executor.request_id, "abc")

    def test_log_name_property(self) -> None:
        """Test if log_name property returns correct value."""
        self.assertEqual(self.executor.log_name, "my_log")

    def test_additional_log_names_property(self) -> None:
        """Test if additional_log_names property returns correct value."""
        self.assertIs(type(self.executor.additional_log_names), list)
        self.assertEqual(
            self.executor.additional_log_names,
            ["additional_log1", "additional_log2"],
        )

    def test_is_not_multi_commands(self) -> None:
        """Test if execution type is recognized correctly."""
        result = self.executor.is_multi_commands(["echo", "Hello world!"])
        self.assertFalse(result)

    def test_is_multi_commands(self) -> None:
        """Test if multi command execution is recognized correctly."""
        result = self.executor.is_multi_commands(
            [
                ["echo", "Hello"],
                ["echo", "world!"],
            ],
        )
        self.assertTrue(result)

    def test_process_call(self) -> None:
        """Test if multi command execution is recognized correctly."""
        print_phrase = "Hello world!"
        proc = self.executor.call(["echo", print_phrase])
        self.assertTrue(proc.is_ok)

        logs = self.executor.additional_log_names
        if self.executor.log_name is not None:
            logs.append(f"{self.executor.log_name}.txt")
        for log in logs:
            with open(os.path.join(self.executor.workdir, log), "r") as log_file:
                self.assertEqual(log_file.readline().rstrip("\n"), print_phrase)


if __name__ == "__main__":
    unittest.main()
